﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms.DataVisualization.Charting;
using System.Xml;

namespace CPXDI_ChartPresenter
{
    public partial class GrafForm : Form
    {
        public GrafForm()
        {
            InitializeComponent();
        }

        // Regex pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^(W|w|B|n|H ?|L ?|O|U|S|s|V|G|g|p)\d+$");

        // Regex pro rozpoznávání událostí
        Regex myRegex2 = new Regex(@"^e(Start|StartWithBP|Stop)(|:*)$");

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "W", "W_set_auto" }, { "w", "W_set_manual" },
            { "B", "W_act" }, { "n", "R_act" }, { "H", "HR_act" }, { "L", "SpO2_act" }, { "O", "BPS_act" }, { "U", "DBP_act" },
            { "V", "Speed_act" }, { "p", "Slope_act" }, { "S", "Speed_set_auto" }, { "s", "Speed_set_manual" }, { "G", "Slope_set_auto" },
            { "g", "Slope_set_manual" } };

        bool merime = false;
        bool pokracuj = false;
        bool updateChart = false;
        TcpListener listener;
        TcpClient client;
        NetworkStream clientStream;
        Thread listenerThread;

        Encoding encoding = Encoding.ASCII; // použité kódování
        const int BUFSIZE = 32; // maximální velikost přijímané zprávy
        const string NEWLINE = "\r"; // znak pro konec zprávy

        IPAddress ipAdresa;
        int cisloPortu;
        int newLineCode;
        byte[] rcvBuffer = new byte[BUFSIZE]; // přijímací buffer
        int bytesRcvd;
        int nextByte;
        /// <summary>
        /// zajišťuje vytvoření TCP serveru a poslouchání zpráv
        /// </summary>
        void DoListening()
        {
            if (!IPAddress.TryParse(ipAdresaTextBox.Text, out ipAdresa))
            {
                pokracuj = false;
                DisplayData("Neplatná IP adresa");
            }

            if (!int.TryParse(portTextBox.Text, out cisloPortu))
            {
                pokracuj = false;
                DisplayData("Neplatné číslo portu");
            }

            while (pokracuj)
            {
                try
                {
                    //vytvorime intanci posluchace pro urcity TCP port
                    listener = new TcpListener(ipAdresa, cisloPortu);
                    DisplayData("Čekám na připojení od: " + listener.LocalEndpoint);
                    //zacneme naslouchani na urcenem portu
                    listener.Start();
                    client = null;
                    clientStream = null;
                    //pockame na pripojeni nejakeho klienta
                    client = listener.AcceptTcpClient();
                    DisplayData("Připojil se klient");
                    //po pripojeni si vyzvedneme proud
                    clientStream = client.GetStream();

                    newLineCode = encoding.GetBytes(NEWLINE)[0];
                    bytesRcvd = 0;
                    while ((nextByte = clientStream.ReadByte()) > -1)
                    {
                        if (nextByte == newLineCode)
                        {
                            fronta.EnqueueTask(encoding.GetString(rcvBuffer, 0, bytesRcvd));
                            bytesRcvd = 0;
                        }
                        else if (bytesRcvd < BUFSIZE)
                        {
                            rcvBuffer[bytesRcvd] = (byte)nextByte;
                            bytesRcvd++;
                        }
                        else
                        {
                            fronta.EnqueueTask(encoding.GetString(rcvBuffer, 0, bytesRcvd));
                            bytesRcvd = 0;
                            rcvBuffer[bytesRcvd] = (byte)nextByte;
                            bytesRcvd++;
                        }
                    }

                    DisplayData("Klient se odpojil");
                    clientStream.Close();
                    client.Close();
                    listener.Stop();
                }
                catch (SocketException ex)
                {
                    DisplayData(ex.Message);
                }
                catch (IOException ex)
                {
                    DisplayData(ex.Message);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        /// <summary>
        /// Metoda pro pomocné zobrazení (log)
        /// </summary>
        /// <param name="msg">zpráva k zobrazení</param>
        private void DisplayData(string msg)
        {
            logTextBox.Invoke(new EventHandler(delegate
            {
                logTextBox.AppendText(msg + "\n");
            }));
        }

        TaskQueue<string> fronta;
        /// <summary>
        /// start a stop měření (TCP serveru)
        /// </summary>
        private void startStopButton_Click(object sender, EventArgs e)
        {
            if (!merime) // úkony pro spuštění
            {
                fronta = new TaskQueue<string>(this);

                pokracuj = true;
                startStopButton.Text = "Zastavit";
                merime = true;
                if (!automaticUdalosti.Checked) { updateChart = true; }
                listenerThread = new Thread(DoListening);
                listenerThread.Start();
            }
            else // úkony pro zastavení
            {
                fronta.Dispose();

                pokracuj = false;
                try
                {
                    if (clientStream != null)
                    {
                        clientStream.Close();
                    }
                    if (client != null)
                    {
                        client.Close();
                    }
                    if (listener != null)
                    {
                        listener.Stop();
                    }
                }
                catch (Exception ex)
                {
                    DisplayData(ex.Message);
                }
                startStopButton.Text = "Spustit";
                merime = false;
                updateChart = false;
            }
        }

        /// <summary>
        /// při zavírání okna se ukončí měření (pokud ještě probíhá)
        /// </summary>
        private void GrafForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (merime)
            {
                startStopButton_Click(sender, e);
                MessageBox.Show("Běžící TCP spojení zastaveno", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        byte[] byteBuffer;
        /// <summary>
        /// zajišťuje odesílání dat (zápis do Network Streamu) - příkaz má být bez koncového znaku, koncový znak se přidá před zapsáním
        /// </summary>
        /// <param name="data">odesílaný příkaz (bez koncového znaku)</param>
        void SendData(string data)
        {
            try
            {
                // Convert input String to bytes
                byteBuffer = encoding.GetBytes(data + NEWLINE);

                // Send the encoded string to the server
                clientStream.Write(byteBuffer, 0, byteBuffer.Length);
            }
            catch (Exception ex)
            {
                DisplayData("Chyba při odesílání: " + ex.Message);
            }
        }

        /// <summary>
        /// zajišťuje přidání bodu do grafu
        /// </summary>
        /// <param name="series">série, do které chceme bod přidat</param>
        /// <param name="dateTime">hodnota bodu na ose X</param>
        /// <param name="hodnota">hodnota bodu na ose Y</param>
        private void addDataPoint(Series series, DateTime dateTime, double hodnota)
        {
            mainChart.Invoke(new EventHandler(delegate
            {
                // Add data point to the series
                series.Points.AddXY(dateTime, hodnota);
            }));
        }

        /// <summary>
        /// kontroluje automatické posunování grafu
        /// </summary>
        /// <param name="maximum">hodnota maxima na ose X</param>
        private void checkScroll(double maximum)
        {
            mainChart.Invoke(new EventHandler(delegate
            {
                if (mainChartArea.AxisX.Maximum < maximum)
                {
                    // Set manual maximum value
                    mainChartArea.AxisX.Maximum = maximum;

                    if (autoScrollCheckBox.Checked)
                    {
                        mainChartArea.AxisX.ScaleView.Zoom(DateTime.FromOADate(maximum).AddSeconds(-30).ToOADate(), maximum);
                    }
                }
            }));
        }

        DateTime pomDateTime;
        double pomMax;
        int result;
        /// <summary>
        /// zajišťuje zpracování příkazů (ty dostává z fronty úkolů)
        /// </summary>
        /// <param name="data">příkaz ke zpracování</param>
        public void ZpracujData(string data)
        {
            // přišla hodnota veličiny
            if (myRegex1.IsMatch(data))
            {
                if (updateChart)
                {
                    pomDateTime = DateTime.Now;
                    if (int.TryParse(data.Substring(1), out result))
                    {
                        // Add data point to the series
                        addDataPoint(prikazySeries[data.Substring(0, 1)], pomDateTime, result);
                    }

                    pomMax = pomDateTime.AddMilliseconds(1000 - pomDateTime.Millisecond).ToOADate();
                    checkScroll(pomMax);
                }
            }
            // přišla událost
            else if (myRegex2.IsMatch(data))
            {
                if (data == "eStart") 
                {
                    DisplayData("Událost: Start");
                    if (automaticUdalosti.Checked)
                    {
                        vymazatButton.Invoke(new EventHandler(vymazatButton_Click));
                        updateChart = true;
                    }
                }
                else if (data == "eStartWithBP")
                {
                    DisplayData("Událost: StartWithBP");
                    if (automaticUdalosti.Checked)
                    {
                        vymazatButton.Invoke(new EventHandler(vymazatButton_Click));
                        updateChart = true;
                    }
                }
                else if (data == "eStop")
                {
                    DisplayData("Událost: Stop");
                    if (automaticUdalosti.Checked)
                    {
                        // TODO
                        updateChart = false;
                    }
                }
                else
                {
                    DisplayData("Neobsloužená událost: " + data);
                }
            }
            // nerozpoznáno
            else
            {
                DisplayData("Nerozpoznáno: " + data);
            }
        }

        ChartArea mainChartArea;
        Legend mainLegend;
        Dictionary<string, Series> prikazySeries = new Dictionary<string, Series>();
        /// <summary>
        /// načtení nového grafu
        /// </summary>
        private void GrafForm_Load(object sender, EventArgs e)
        {
            // Create Chart Area
            mainChartArea = new ChartArea();

            mainChart.ChartAreas.Clear();
            // Add Chart Area to the chart
            mainChart.ChartAreas.Add(mainChartArea);

            mainChartArea.AxisX.Interval = 1;
            mainChartArea.AxisX.IntervalType = DateTimeIntervalType.Seconds;
            mainChartArea.AxisX.ScaleView.SmallScrollSize = 1;
            mainChartArea.AxisX.ScaleView.SmallScrollSizeType = DateTimeIntervalType.Seconds;
            // Show small scroll buttons only
            mainChartArea.AxisX.ScrollBar.ButtonStyle = ScrollBarButtonStyles.SmallScroll;
            // Set X axis labels format
            mainChartArea.AxisX.LabelStyle.Format = "HH:mm:ss";

            DateTime dt = DateTime.Now;
            dt = dt.AddMilliseconds(-dt.Millisecond);
            // Set manual minimum and maximum values
            mainChartArea.AxisX.Minimum = dt.ToOADate();
            mainChartArea.AxisX.Maximum = dt.AddSeconds(30).ToOADate();

            // Enable scale breaks
            mainChartArea.AxisY.ScaleBreakStyle.Enabled = true;

            // Create Legend
            mainLegend = new Legend("MainLegend");

            mainChart.Legends.Clear();
            // Add Legend to the chart
            mainChart.Legends.Add(mainLegend);

            // Set legend style
            mainLegend.LegendStyle = LegendStyle.Table;

            // Set table style if legend style is Table
            mainLegend.TableStyle = LegendTableStyle.Tall;

            // Set legend docking
            mainLegend.Docking = Docking.Bottom;

            // Set legend alignment
            mainLegend.Alignment = StringAlignment.Near;

            mainLegend.MaximumAutoSize = 25;

            mainChart.Series.Clear();
            prikazySeries.Clear();
            foreach (var item in prikazyId)
            {
                // Create a data series
                prikazySeries.Add(item.Key, new Series(item.Value));

                // Add series to the chart
                mainChart.Series.Add(prikazySeries[item.Key]);

                // Set series chart type
                prikazySeries[item.Key].ChartType = SeriesChartType.Line;

                prikazySeries[item.Key].XValueType = ChartValueType.DateTime;

                // Set marker attributes for the whole series
                prikazySeries[item.Key].MarkerStyle = MarkerStyle.Circle;

                // Set ToolTips for Data Point Series
                prikazySeries[item.Key].ToolTip = "#VALX{HH:mm:ss}: #VAL";

                // Associate series to the Legend
                prikazySeries[item.Key].Legend = "MainLegend";
            }
        }

        /// <summary>
        /// vymaže graf (načte nový)
        /// </summary>
        private void vymazatButton_Click(object sender, EventArgs e)
        {
            GrafForm_Load(sender, e);
        }

        /// <summary>
        /// exportuje data grafu do XML
        /// </summary>
        private void exportButton_Click(object sender, EventArgs e)
        {
            // Export chart series values into the data set
            DataSet dataSet = mainChart.DataManipulator.ExportSeriesValues();
            dataSet.DataSetName = "Export_" + DateTime.Now.ToString("d.M.yyyy_HH-mm-ss");

            // Get XML data and schema from data set
            string xmlString = dataSet.GetXml();

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(xmlString);
            DialogResult dr = mainSaveFileDialog.ShowDialog();
            if (dr == DialogResult.OK)
            {
                xmlDoc.Save(mainSaveFileDialog.FileName);
            }
        }

        private void automaticUdalosti_CheckedChanged(object sender, EventArgs e)
        {
            if (!automaticUdalosti.Checked)
            {
                updateChart = true;
            }
        }
    }
}
