﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text.RegularExpressions;

namespace CPXDI_ManualController
{
    public partial class OvladaniForm : Form
    {
        public OvladaniForm()
        {
            InitializeComponent();

            vykonGroupBox.Enabled = false;
            rychlostGroupBox.Enabled = false;
            sklonGroupBox.Enabled = false;
            udalostGroupBox.Enabled = false;
        }

        bool merime = false;
        bool pokracuj = false;
        TcpListener listener;
        TcpClient client;
        NetworkStream clientStream;
        Thread listenerThread;

        Encoding encoding = Encoding.ASCII; // použité kódování
        const int BUFSIZE = 32; // maximální velikost přijímané zprávy
        const string NEWLINE = "\r"; // znak pro konec zprávy

        IPAddress ipAdresa;
        int cisloPortu;
        int newLineCode;
        byte[] rcvBuffer = new byte[BUFSIZE]; // přijímací buffer
        int bytesRcvd;
        int nextByte;
        /// <summary>
        /// zajišťuje vytvoření TCP serveru a poslouchání zpráv
        /// </summary>
        void DoListening()
        {
            if (!IPAddress.TryParse(ipAdresaTextBox.Text, out ipAdresa))
            {
                pokracuj = false;
                DisplayData("Neplatná IP adresa");
            }

            if (!int.TryParse(portTextBox.Text, out cisloPortu))
            {
                pokracuj = false;
                DisplayData("Neplatné číslo portu");
            }

            while (pokracuj)
            {
                try
                {
                    //vytvorime intanci posluchace pro urcity TCP port
                    listener = new TcpListener(ipAdresa, cisloPortu);
                    DisplayData("Čekám na připojení od: " + listener.LocalEndpoint);
                    //zacneme naslouchani na urcenem portu
                    listener.Start();
                    client = null;
                    clientStream = null;
                    //pockame na pripojeni nejakeho klienta
                    client = listener.AcceptTcpClient();
                    DisplayData("Připojil se klient");
                    //aktivujeme ovládací prvky
                    nastavGroupBoxy(true);
                    //po pripojeni si vyzvedneme proud
                    clientStream = client.GetStream();

                    newLineCode = encoding.GetBytes(NEWLINE)[0];
                    bytesRcvd = 0;
                    while ((nextByte = clientStream.ReadByte()) > -1)
                    {
                        if (nextByte == newLineCode)
                        {
                            fronta.EnqueueTask(encoding.GetString(rcvBuffer, 0, bytesRcvd));
                            bytesRcvd = 0;
                        }
                        else if (bytesRcvd < BUFSIZE)
                        {
                            rcvBuffer[bytesRcvd] = (byte)nextByte;
                            bytesRcvd++;
                        }
                        else
                        {
                            fronta.EnqueueTask(encoding.GetString(rcvBuffer, 0, bytesRcvd));
                            bytesRcvd = 0;
                            rcvBuffer[bytesRcvd] = (byte)nextByte;
                            bytesRcvd++;
                        }
                    }

                    DisplayData("Klient se odpojil");
                    //deaktivujeme ovládací prvky
                    nastavGroupBoxy(false);

                    clientStream.Close();
                    client.Close();
                    listener.Stop();
                }
                catch (SocketException ex)
                {
                    DisplayData(ex.Message);
                }
                catch (IOException ex)
                {
                    DisplayData(ex.Message);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        /// <summary>
        /// metoda pro pomocné zobrazení (log)
        /// </summary>
        /// <param name="msg">zpráva k zobrazení</param>
        private void DisplayData(string msg)
        {
            logTextBox.Invoke(new EventHandler(delegate
            {
                logTextBox.AppendText(msg + "\n");
            }));
        }

        /// <summary>
        /// metoda pro zobrazení přijaté hodnoty Auto
        /// </summary>
        /// <param name="value">hodnota k zobrazení</param>
        /// <param name="label">Label, do kterého se má zobrazit</param>
        private void DisplayValue(string value, Label label)
        {
            label.Invoke(new EventHandler(delegate
            {
                label.Text = value;
            }));
        }

        /// <summary>
        /// nastavení stavu pro daný GroupBox
        /// </summary>
        /// <param name="state">požadovaný stav</param>
        /// <param name="groupBox">GroupBox, který chceme nastavit</param>
        private void SetEnabledState(bool state, GroupBox groupBox)
        {
            groupBox.Invoke(new EventHandler(delegate
            {
                groupBox.Enabled = state;
            }));
        }

        TaskQueue<string> fronta;
        /// <summary>
        /// start a stop měření (TCP serveru)
        /// </summary>
        private void startStopButton_Click(object sender, EventArgs e)
        {
            if (!merime) // úkony pro spuštění
            {
                fronta = new TaskQueue<string>(this);
                
                pokracuj = true;
                startStopButton.Text = "Zastavit";
                merime = true;
                listenerThread = new Thread(DoListening);
                listenerThread.Start();
            }
            else // úkony pro zastavení
            {
                fronta.Dispose();
                
                pokracuj = false;
                try
                {
                    if (clientStream != null)
                    {
                        clientStream.Close();
                    }
                    if (client != null)
                    {
                        client.Close();
                    }
                    if (listener != null)
                    {
                        listener.Stop();
                    }
                }
                catch (Exception ex)
                {
                    DisplayData(ex.Message);
                }
                //deaktivujeme ovládací prvky
                nastavGroupBoxy(false);

                startStopButton.Text = "Spustit";
                merime = false;
            }
        }

        private void OvladaniForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (merime)
            {
                startStopButton_Click(sender, e);
                MessageBox.Show("Běžící TCP spojení zastaveno", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        byte[] byteBuffer;
        /// <summary>
        /// zajišťuje odesílání dat (zápis do Network Streamu) - příkaz má být bez koncového znaku, koncový znak se přidá před zapsáním
        /// </summary>
        /// <param name="data">odesílaný příkaz (bez koncového znaku)</param>
        void SendData(string data)
        {
            try
            {
                // Convert input String to bytes
                byteBuffer = encoding.GetBytes(data + NEWLINE);

                // Send the encoded string to the server
                clientStream.Write(byteBuffer, 0, byteBuffer.Length);
            }
            catch (Exception ex)
            {
                DisplayData("Chyba při odesílání: " + ex.Message);
            }
        }

        Regex myRegex1 = new Regex(@"^W\d{3}$");
        Regex myRegex2 = new Regex(@"^(S|G)\d{4}$");
        Regex myRegex3 = new Regex(@"^(s|P|F)$");
        /// <summary>
        /// zajišťuje zpracování příkazů (ty dostává z fronty úkolů)
        /// </summary>
        /// <param name="data">příkaz ke zpracování</param>
        public void ZpracujData(string data)
        {
            // vykon kola
            if (myRegex1.IsMatch(data))
            {
                // zobrazit hodnotu vykonu kola
                DisplayValue(data.Substring(1), vykonAutoLabel);
                // v automatickem rezimu odeslat hodnotu jako manual
                if (automaticVykon.Checked)
                {
                    SendData("W" + vykonAutoLabel.Text);
                }
            }
            // rychlost a sklon behatka
            else if (myRegex2.IsMatch(data))
            {
                switch (data.Substring(0,1))
                {
                    case "S":
                        // zobrazit hodnotu rychlosti
                        DisplayValue(data.Substring(1), rychlostAutoLabel);
                        // v automatickem rezimu odeslat hodnotu jako manual
                        if (automaticRychlost.Checked)
                        {
                            SendData("S" + rychlostAutoLabel.Text);
                        }
                        break;
                    case "G":
                        // zobrazit hodnotu sklonu
                        DisplayValue(data.Substring(1), sklonAutoLabel);
                        // v automatickem rezimu odeslat hodnotu jako manual
                        if (automaticSklon.Checked)
                        {
                            SendData("G" + sklonAutoLabel.Text);
                        }
                        break;
                    default:
                        DisplayData("Neošetřené nastavení běhátka: " + data);
                        break;
                }
            }
            // udalosti
            else if (myRegex3.IsMatch(data))
            {
                switch (data.Substring(0, 1))
                {
                    case "s":
                        DisplayData("Událost: Start");
                        if (startUdalostButton.Enabled)
                        {
                            startUdalostButton.Invoke(new EventHandler(zmenaStavuUdalostiButton));
                        }
                        break;
                    case "P":
                        DisplayData("Událost: StartWithBP");
                        if (startWithBPUdalostButton.Enabled) 
                        {
                            startWithBPUdalostButton.Invoke(new EventHandler(zmenaStavuUdalostiButton));
                        }
                        break;
                    case "F":
                        DisplayData("Událost: Stop");
                        if (stopUdalostButton.Enabled)
                        {
                            stopUdalostButton.Invoke(new EventHandler(zmenaStavuUdalostiButton));
                        }
                        break;
                    default:
                        DisplayData("Neošetřená událost: " + data);
                        break;
                }
            }
            else
            {
                DisplayData("Nerozpoznáno: " + data);
            }
        }

        /// <summary>
        /// obsluha tlačítek pro přenesení hodnoty Auto na hodnotu Manual
        /// </summary>
        void obsluzPrenestButton(object sender, EventArgs e)
        {
            if (sender == vykonPrenestButton)
            {
                if (myRegex1.IsMatch("W" + vykonAutoLabel.Text))
                {
                    vykonManualTextBox.Text = vykonAutoLabel.Text;
                }
            }
            else if (sender == rychlostPrenestButton)
            {
                if (myRegex2.IsMatch("S" + rychlostAutoLabel.Text))
                {
                    rychlostManualTextBox.Text = rychlostAutoLabel.Text;
                }
            }
            else if (sender == sklonPrenestButton)
            {
                if (myRegex2.IsMatch("G" + sklonAutoLabel.Text))
                {
                    sklonManualTextBox.Text = sklonAutoLabel.Text;
                }
            }
        }

        /// <summary>
        /// obsluha změny stavu tlačítek událostí
        /// </summary>
        void zmenaStavuUdalostiButton(object sender, EventArgs e)
        {
            if (sender == startUdalostButton)
            {
                if (AutomaticEvents.Checked)
                { 
                    stopUdalostButton.Enabled = true;
                    startUdalostButton.Enabled = false;
                    startWithBPUdalostButton.Enabled = false;
                }
            }
            else if (sender == startWithBPUdalostButton)
            {
                if (AutomaticEvents.Checked)
                {
                    stopUdalostButton.Enabled = true;
                    startUdalostButton.Enabled = false;
                    startWithBPUdalostButton.Enabled = false;
                }
            }
            else if (sender == stopUdalostButton)
            {
                if (AutomaticEvents.Checked)
                {
                    stopUdalostButton.Enabled = false;
                    startUdalostButton.Enabled = true;
                    startWithBPUdalostButton.Enabled = true;
                }
            }
        }

        /// <summary>
        /// obsluha odesílání nastavených hodnot Manual a událostí
        /// </summary>
        void obsluzOdeslatButton(object sender, EventArgs e)
        {
            if (sender == vykonOdeslatButton || sender == vykonPlusButton || sender == vykonMinusButton)
            {
                if (vykonManualTextBox.Text.Length < 3)
                {
                    vykonManualTextBox.Text = vykonManualTextBox.Text.PadLeft(3, '0');
                }
                SendData("W" + vykonManualTextBox.Text);
            }
            else if (sender == rychlostOdeslatButton || sender == rychlostPlusButton || sender == rychlostMinusButton)
            {
                if (rychlostManualTextBox.Text.Length < 4)
                {
                    rychlostManualTextBox.Text = rychlostManualTextBox.Text.PadLeft(4, '0');
                }
                SendData("S" + rychlostManualTextBox.Text);
            }
            else if (sender == sklonOdeslatButton || sender == sklonPlusButton || sender == sklonMinusButton)
            {
                if (sklonManualTextBox.Text.Length < 4)
                {
                    sklonManualTextBox.Text = sklonManualTextBox.Text.PadLeft(4, '0');
                }
                SendData("G" + sklonManualTextBox.Text);
            }
            else if (sender == startUdalostButton)
            {
                SendData("s");
                zmenaStavuUdalostiButton(sender, e);
            }
            else if (sender == startWithBPUdalostButton)
            {
                SendData("P");
                zmenaStavuUdalostiButton(sender, e);
            }
            else if (sender == stopUdalostButton)
            {
                SendData("F");
                zmenaStavuUdalostiButton(sender, e);
            }
            else if (sender == remarkUdalostButton)
            {
                SendData("R" + remarkTextBox.Text);
            }
        }

        int puvodni;
        TextBox puvodniTextBox;
        int krok;
        string krokString;
        int nova;
        string novaString;
        int horniMez;
        bool ok;
        /// <summary>
        /// obsluha tlačítek + a -
        /// </summary>
        void obsluzKrok(object sender, EventArgs e)
        {
            ok = false;
            if (sender == vykonPlusButton || sender == vykonMinusButton)
            {
                puvodniTextBox = vykonManualTextBox;
                krokString = vykonKrokTextBox.Text;
                horniMez = 1000;
            }
            else if ((sender == rychlostPlusButton || sender == rychlostMinusButton) && rychlostKrokTextBox.Text != "")
            {
                puvodniTextBox = rychlostManualTextBox;
                if (puvodniTextBox.Text != "")
                {
                    puvodniTextBox.Text = puvodniTextBox.Text.Remove(puvodniTextBox.Text.Length - 1);
                }
                krokString = rychlostKrokTextBox.Text.Remove(rychlostKrokTextBox.Text.Length - 1);
                horniMez = 256;
            }
            else if ((sender == sklonPlusButton || sender == sklonMinusButton) && sklonKrokTextBox.Text != "")
            {
                puvodniTextBox = sklonManualTextBox;
                if (puvodniTextBox.Text != "")
                {
                    puvodniTextBox.Text = puvodniTextBox.Text.Remove(puvodniTextBox.Text.Length - 1);
                }
                krokString = sklonKrokTextBox.Text.Remove(sklonKrokTextBox.Text.Length - 1);
                horniMez = 256;
            }
            else
            {
                puvodniTextBox = new TextBox();
                puvodniTextBox.Text = "o";
                krokString = "o";
                horniMez = 0;
            }
            
            if (int.TryParse(puvodniTextBox.Text, out puvodni))
            {
                if (int.TryParse(krokString, out krok))
                {
                    if (sender == vykonPlusButton || sender == rychlostPlusButton || sender == sklonPlusButton)
                    {
                        nova = puvodni + krok;
                    }
                    else if (sender == vykonMinusButton || sender == rychlostMinusButton || sender == sklonMinusButton)
                    {
                        nova = puvodni - krok;
                    }
                    else
                    {
                        nova = -1;
                    }
                    
                    if (nova >= 0 && nova < horniMez)
                    {
                        novaString = nova.ToString();
                        if (horniMez == 256)
                        {
                            novaString += "0";
                        }
                        puvodniTextBox.Text = novaString;
                        obsluzOdeslatButton(sender, e);
                        ok = true;
                    }
                }
            }

            if (!ok && horniMez == 256)
            {
                puvodniTextBox.Text += "0";
            }
        }

        private void automaticVykon_CheckedChanged(object sender, EventArgs e)
        {
            vykonKrokTextBox.ReadOnly = automaticVykon.Checked;
            vykonPlusButton.Enabled = !automaticVykon.Checked;
            vykonMinusButton.Enabled = !automaticVykon.Checked;
            vykonManualTextBox.ReadOnly = automaticVykon.Checked;
            vykonPrenestButton.Enabled = !automaticVykon.Checked;
            vykonOdeslatButton.Enabled = !automaticVykon.Checked;
        }

        private void automaticRychlost_CheckedChanged(object sender, EventArgs e)
        {
            rychlostKrokTextBox.ReadOnly = automaticRychlost.Checked;
            rychlostPlusButton.Enabled = !automaticRychlost.Checked;
            rychlostMinusButton.Enabled = !automaticRychlost.Checked;
            rychlostManualTextBox.ReadOnly = automaticRychlost.Checked;
            rychlostPrenestButton.Enabled = !automaticRychlost.Checked;
            rychlostOdeslatButton.Enabled = !automaticRychlost.Checked;
        }

        private void automaticSklon_CheckedChanged(object sender, EventArgs e)
        {
            sklonKrokTextBox.ReadOnly = automaticSklon.Checked;
            sklonPlusButton.Enabled = !automaticSklon.Checked;
            sklonMinusButton.Enabled = !automaticSklon.Checked;
            sklonManualTextBox.ReadOnly = automaticSklon.Checked;
            sklonPrenestButton.Enabled = !automaticSklon.Checked;
            sklonOdeslatButton.Enabled = !automaticSklon.Checked;
        }

        private void AutomaticEvents_CheckedChanged(object sender, EventArgs e)
        {
            // nahodit automaticky rezim
            if (AutomaticEvents.Checked)
            {
                startUdalostButton.Enabled = true;
                startWithBPUdalostButton.Enabled = true;
                stopUdalostButton.Enabled = false;
            }
            // nahodit manualni rezim
            else
            {
                startUdalostButton.Enabled = true;
                startWithBPUdalostButton.Enabled = true;
                stopUdalostButton.Enabled = true;
            }
        }

        private void nastavGroupBoxy(bool stav)
        {
            SetEnabledState(stav, vykonGroupBox);
            SetEnabledState(stav, rychlostGroupBox);
            SetEnabledState(stav, sklonGroupBox);
            SetEnabledState(stav, udalostGroupBox);
        }
    }
}
