﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class BtlEcg_Slave : Protokol
    {
        // Regex pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^HR=\d{3}$");

        // Vazba mezi "příkazy" a veličinami pro rozpoznávání příkazů (Response from BtlEcg)
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "1", "HR_act" } };
        
        public override string VypisJmeno()
        {
            return "BtlEcg_Slave";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.BaudRate = 9600;
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.One;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            zarizeni.Casovac.Interval = 10000;
            zarizeni.pouzivatTimer = true;

            stariPrijateHodnoty = DateTime.MinValue;
        }

        Velicina velicina2;
        DateTime stariPrijateHodnoty = DateTime.MinValue;
        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            if (stariPrijateHodnoty != DateTime.MinValue)
            {
                // přidá do fronty mazací příkaz (hodnotu null) v případě, že poslední přijatá hodnota je starší než 10 sekund
                if ((DateTime.Now - stariPrijateHodnoty).Seconds > 10)
                {
                    velicina2 = Velicina.vyhledejVelicinu(prikazyId["1"]); // to je vyhledání HR_act
                    // pokud rozpozná veličinu (TF), vloží ji s hodnotou null do fronty pro globální monitoring
                    if (velicina2 != null)
                    {
                        MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina2, null, zarizeni));
                        stariPrijateHodnoty = DateTime.MinValue;
                    }
                }
            }
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            return "nepodporováno";
        }

        Velicina velicina;
        string ret;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexem: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                // ukládání hodnot
                ret = "";
                velicina = Velicina.vyhledejVelicinu(prikazyId["1"]); // to je vyhledání HR_act
                // pokud rozpozná veličinu (TF), vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(3, 3), odkud));
                    stariPrijateHodnoty = DateTime.Now;
                }
                else
                {
                    ret += "nerozpoznáno Id tep.frekvence";
                }
                return ret;
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return null;
        }
    }
}
