﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class Ergoline_Master : Protokol
    {
        // Regexy pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^W\d{3}$"); // pro Požadovaný výkon
        Regex myRegex2 = new Regex(@"^(B|D|H|O|U)$"); // pro odpovědi
        Regex myRegex3 = new Regex(@"^(S|s|F)$"); // pro události: I a K nyní ignorujeme

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů (Command from PC / ECG)
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "W", "W_set_auto" }, { "B", "W_act" }, { "D", "R_act" },
            { "H", "HR_act" }, { "O", "BPS_act" }, { "U", "DBP_act" } };

        // Vazba mezi veličinami a příkazy pro posílání odpovědí (Response)
        Dictionary<string, string> idPrikazy = new Dictionary<string, string>() { { "W_act", "B" }, { "R_act", "n" }, { "HR_act", "H" },
            { "BPS_act", "O" }, { "DBP_act", "U" } };

        // Vazba mezi příkazy a událostmi pro rozpoznávání příkazů událostí (Command from PC / ECG)
        Dictionary<string, string> prikazyUdalostiId = new Dictionary<string, string>() { { "S", "start_with_BP" }, { "s", "start" },
            { "F", "stop" } };
        
        public override string VypisJmeno()
        {
            return "Ergoline_Master";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.BaudRate = 4800; // lze i 1200, 2400, 4800, 9600, 19200, 38400
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.One;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }
        
        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            // Timer se nepoužívá
        }

        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            // nepoužívá se
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            return "nepodporováno";
        }

        Velicina velicina;
        string hodnota;
        Udalost udalost;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexy: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz.Substring(0, 1)]);
                // pokud rozpozná veličinu odpovídající W (Požadovaný výkon), vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(1), odkud)); // TODO: W_set_auto tu bylo natvrdo v kodu ... Opraveno.
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else if (myRegex2.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz]);
                // pokud rozpozná veličinu odpovídající B, D, H, O, U, tak posílá odpověď na dotaz
                if (velicina != null)
                {
                    try
                    {
                        hodnota = MereniForm.vratAktVelicinyHodnoty()[velicina];
                    }
                    catch (Exception)
                    {
                        hodnota = "000";
                    }
                    if (hodnota.Length > 3)
                    {
                        hodnota = "000";
                    }
                    if (hodnota.Length < 3)
                    {
                        hodnota = hodnota.PadLeft(3, '0');
                    }
                    odkud.OdesliPrikaz(idPrikazy[velicina.Id] + hodnota);
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else if (myRegex3.IsMatch(prikaz))
            {
                udalost = Udalost.vyhledejUdalost(prikazyUdalostiId[prikaz]);
                // pokud rozpozná událost odpovídající S, s, F, tak ji vloží do fronty událostí
                if (udalost != null)
                {
                    MereniForm.vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(udalost, "", odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id události";
                }
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return null;
        }
    }
}
