﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class Ergoline_Slave : Protokol
    {
        // Regex pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^(B|n|H|O|U)\d{3}$");

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů (Response from Ergoselect)
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "B", "W_act" }, { "n", "R_act" }, { "H", "HR_act" },
            { "O", "BPS_act" }, { "U", "DBP_act" } };

        // Vazba mezi veličinami a příkazy pro posílání hodnot nebo dotazů (Command from PC)
        Dictionary<string, string> idPrikazy = new Dictionary<string, string>() { { "W_set_auto", "W" }, { "W_set_manual", "W" },
            { "W_act", "B" }, { "R_act", "D" }, { "HR_act", "H" }, { "BPS_act", "O" }, { "DBP_act", "U" } };
        
        public override string VypisJmeno()
        {
            return "Ergoline_Slave";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.BaudRate = 4800; // lze i 1200, 2400, 4800, 9600, 19200, 38400
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.One;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            zarizeni.pouzivatTimer = true;

            init = true;
        }

        Scenar scenar;
        List<Velicina> sezVel = new List<Velicina>();
        bool init = true;
        int i = -1;
        Velicina VydejVelicinu(Zarizeni zarizeni)
        {
            if (init)
            {
                scenar = MereniForm.vratAktScenar();
                if (scenar != null)
                {
                    sezVel = scenar.VstupniVelicinyProZarizeni[zarizeni].Concat(scenar.VystupniVelicinyProZarizeni[zarizeni]).ToList();
                    i = -1;
                    init = false;
                }
            }

            if (!init && sezVel.Count > 0)
            {
                i++;
                if (i == sezVel.Count-1)
                {
                    init = true;
                }
                return sezVel[i];
            }
            else
            {
                return null;
            }
        }

        Velicina vydana;
        string hodnota;
        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            vydana = VydejVelicinu(zarizeni);
            if (vydana != null)
            {
                if (vydana.Id == "W_set_auto" || vydana.Id == "W_set_manual") // posílání hodnoty Požadovaného výkonu
                {
                    try
                    {
                        hodnota = MereniForm.vratAktVelicinyHodnoty()[vydana];
                    }
                    catch (Exception)
                    {
                        hodnota = "";
                    }
                    if (hodnota.Length > 3)
                    {
                        hodnota = "";
                    }
                    if (hodnota.Length > 0 && hodnota.Length < 3)
                    {
                        hodnota = hodnota.PadLeft(3, '0');
                    }
                    if (hodnota == "" && zarizeni.posilatNeexistujiciHodnoty)
                    {
                        hodnota = "000";
                    }
                    if (hodnota != "")
                    {
                        zarizeni.OdesliPrikaz(idPrikazy[vydana.Id] + hodnota);
                    }
                }
                else // posílání dotazů na vybrané výstupní veličiny daného zařízení
                {
                    if (idPrikazy.ContainsKey(vydana.Id))
                    {
                        zarizeni.OdesliPrikaz(idPrikazy[vydana.Id]);
                    }
                }
            }
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            switch (udalost.Id)
            {
                case "start":
                    komu.OdesliPrikaz("s", 100);
                    return "";
                case "start_with_BP":
                    komu.OdesliPrikaz("S", 100);
                    return "";
                case "stop":
                    komu.OdesliPrikaz("F", 1000);
                    komu.OdesliPrikaz("F", 1000);
                    komu.OdesliPrikaz("F", 1000);
                    return "";
                default:
                    return "Nepodporovaná událost: " + udalost.Id;
            }
        }

        Velicina velicina;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexem: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz.Substring(0, 1)]);
                // pokud rozpozná veličinu, vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(1), odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        Dictionary<string, string> slovnikPrikazu = new Dictionary<string, string>() { { "START S", "S" }, { "start", "s" }, { "Stop", "F" } };
        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return slovnikPrikazu;
        }
    }
}
