using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class ManualControl_Master : Protokol
    {
        // Regexy pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^W\d{3}$");
        Regex myRegex2 = new Regex(@"^(S|G)\d{4}$");
        Regex myRegex3 = new Regex(@"^(P|s|F|R)$");

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "W", "W_set_manual" }, { "S", "Speed_set_manual" },
            { "G", "Slope_set_manual" } };

        // Vazba mezi veličinami a příkazy pro posílání hodnot
        Dictionary<string, string> idPrikazy = new Dictionary<string, string>() { { "W_set_auto", "W" }, { "Speed_set_auto", "S" },
            { "Slope_set_auto", "G" } };

        // Vazba mezi příkazy a událostmi pro rozpoznávání příkazů událostí
        Dictionary<string, string> prikazyUdalostiId = new Dictionary<string, string>() { { "P", "start_with_BP" }, { "s", "start" },
            { "F", "stop" }, { "R", "remark" } };

        public override string VypisJmeno()
        {
            return "ManualControl_Master";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.Encoding = Encoding.ASCII;
            vlastnosti.BufferSize = 32;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            zarizeni.pouzivatTimer = true;
        }

        Scenar scenar;
        string hodnota;
        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            scenar = MereniForm.vratAktScenar();
            if (scenar != null)
            {
                if (scenar.VstupniVelicinyProZarizeni != null)
                {
                    foreach (var item in scenar.VstupniVelicinyProZarizeni[zarizeni])
                    {
                        // posílání požadované hodnoty Auto veličiny Požadovaný výkon - příkaz W
                        if (item.Id == "W_set_auto")
                        {
                            try
                            {
                                hodnota = MereniForm.vratAktVelicinyHodnoty()[item];
                            }
                            catch (Exception)
                            {
                                hodnota = "";
                            }
                            if (hodnota.Length > 3)
                            {
                                hodnota = "";
                            }
                            if (hodnota.Length > 0 && hodnota.Length < 3)
                            {
                                hodnota = hodnota.PadLeft(3, '0');
                            }
                            if (hodnota == "" && zarizeni.posilatNeexistujiciHodnoty)
                            {
                                hodnota = "000";
                            }
                            if (hodnota != "")
                            {
                                zarizeni.OdesliPrikaz(idPrikazy[item.Id] + hodnota);
                            }
                        }
                        // posílání požadovaných hodnot Auto veličin rychlost a sklon - příkazy S a G
                        if (item.Id == "Speed_set_auto" || item.Id == "Slope_set_auto")
                        {
                            try
                            {
                                hodnota = MereniForm.vratAktVelicinyHodnoty()[item];
                            }
                            catch (Exception)
                            {
                                hodnota = "0000";
                            }
                            if (hodnota.Length != 4)
                            {
                                hodnota = "0000";
                            }
                            if (hodnota == "0000" && !zarizeni.posilatNeexistujiciHodnoty)
                            {
                                hodnota = "";
                            }
                            if (hodnota != "")
                            {
                                zarizeni.OdesliPrikaz(idPrikazy[item.Id] + hodnota);
                            }
                        }
                    }
                }
            }
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            switch (udalost.Id)
            {
                case "start":
                    komu.OdesliPrikaz("s", 100);
                    return "";
                case "start_with_BP":
                    komu.OdesliPrikaz("P", 100);
                    return "";
                case "stop":
                    komu.OdesliPrikaz("F", 100);
                    return "";
                default:
                    return "Nepodporovaná událost: " + udalost.Id;
            }
        }

        Velicina velicina;
        Udalost udalost;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexy: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz) || myRegex2.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz.Substring(0, 1)]);
                // pokud rozpozná veličinu odpovídající W, S, G, vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(1), odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else if (prikaz.Length > 0 && myRegex3.IsMatch(prikaz.Substring(0, 1)))
            {
                udalost = Udalost.vyhledejUdalost(prikazyUdalostiId[prikaz.Substring(0, 1)]);
                // pokud rozpozná událost odpovídající P, s, F, R, tak ji vloží do fronty událostí
                if (udalost != null)
                {
                    MereniForm.vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(udalost, prikaz.Substring(1), odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id události";
                }
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return null;
        }
    }
}
