﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;
using System.Runtime.Serialization;
using System.Xml;

namespace CPXDI_model
{
    public partial class MereniForm : Form
    {
        public MereniForm()
        {
            InitializeComponent();

            // zpracování agrumentů příkazového řádku
            string[] args = Environment.GetCommandLineArgs();

            // jméno konfiguračného souboru
            for (int i = 0; i < args.Length-1; i++)
            {
                if (args[i] == "-c" && !args[i + 1].StartsWith("-"))
                {
                    jmenoKonfigSouboru = args[i + 1];
                }
            }

            // načtení konfiguračních dat
            try
            {
                using (Stream s = File.OpenRead(jmenoKonfigSouboru))
                    objektSeznamu = (ObjektKSerializaci)dataSerializer.ReadObject(s);
                Velicina.SeznamVelicin = objektSeznamu.SeznamVelicin;
                Udalost.SeznamUdalosti = objektSeznamu.SeznamUdalosti;
                Zarizeni.SeznamZarizeni = objektSeznamu.SeznamZarizeni;
                Scenar.SeznamScenaru = objektSeznamu.SeznamScenaru;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Došlo k chybě při deserializaci seznamů (" + ex.Message + "). Bude nastaveno Modelové zapojení.");
                NastavModeloveZapojeni();
            }
            
            vyberScenarComboBox.DataSource = Scenar.SeznamScenaru;
            udalostRadioButton_CheckedChanged(null, null);
            velicinaRadioButton_CheckedChanged(null, null);
            vyberZarizeniComboBox.DataSource = Zarizeni.SeznamZarizeni;

            // vybrání scénáře
            for (int i = 1; i < args.Length-1; i++)
            {
                if (args[i] == "-o" && !args[i + 1].StartsWith("-"))
                {
                    foreach (var item in vyberScenarComboBox.Items)
                    {
                        if (((Scenar)item).Id == args[i + 1])
                        {
                            vyberScenarComboBox.SelectedItem = item;
                            break;
                        }
                    }
                }
            }

            // start měření
            for (int i = 1; i < args.Length; i++)
            {
                if (args[i] == "-s")
                {
                    startStopButton_Click(null, null);
                    break;
                }
            }

            // provedení událostí
            if (merime)
            {
                for (int i = 1; i < args.Length-1; i++)
                {
                    if (args[i] == "-e" && !args[i + 1].StartsWith("-"))
                    {
                        Udalost u = Udalost.vyhledejUdalost(args[i + 1]);
                        if (aktScenar != null && u != null)
                        {
                            List<Udalost> lu = new List<Udalost>() { u };
                            Zarizeni z = new Zarizeni("Automat", "", null, null, null, null, lu, "");
                            aktScenar.VystupniUdalostiProZarizeni[z] = lu;
                            vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(u, "", z));
                        }
                    }
                }
            }
        }

        // údaje pro serializaci
        string jmenoKonfigSouboru = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "CPXDI", "souborSeznamy.xml");
        DataContractSerializer dataSerializer = new DataContractSerializer(typeof(ObjektKSerializaci), null, int.MaxValue, false, true, null);
        ObjektKSerializaci objektSeznamu;
        XmlWriterSettings settings = new XmlWriterSettings() { Indent = true };

        static TaskQueue<Tuple<Velicina, string, Zarizeni>> FrontaVelicin;
        static Tuple<Velicina, string, Zarizeni> trojice;
        /// <summary>
        /// Ukládá data na globální monitoring. Je volaná z fronty, odkud vybírá trojice (Tuple) Veličina, hodnota a Zařízení. Volitelně
        /// zapisuje do logu. Pokud je Veličina v aktuálním scénáři u daného Zařízení v jeho výstupních veličinách, tak ji s hodnotou uloží.
        /// Pokud je přijatá hodnota null, tak se naopak veličina z monitoringu vymaže.
        /// </summary>
        /// <param name="task">musí být trojice (Tuple): Veličina, stringová hodnota a Zařízení</param>
        public void UlozData(object task)
        {
            trojice = (Tuple<Velicina, string, Zarizeni>)task;
            if (trojice.Item2 != null) // hodnota není null, tedy veličinu zapisujeme na monitoring
            {
                try
                {
                    DisplayData("Od " + trojice.Item3 + ": " + trojice.Item1 + ": " + trojice.Item2 + " " + trojice.Item1.Jednotka /*+ " " + DateTime.Now */);
                }
                catch (Exception)
                {

                }
                if (aktScenar.VystupniVelicinyProZarizeni.ContainsKey(trojice.Item3) && aktScenar.VystupniVelicinyProZarizeni[trojice.Item3].Contains(trojice.Item1))
                {
                    aktVelicinyHodnoty[trojice.Item1] = trojice.Item2;
                    try
                    {
                        DisplayMonitorData();
                    }
                    catch (Exception)
                    {

                    }
                }
                else
                {
                    try
                    {
                        DisplayData("není ve výstupních veličinách, zahozeno");
                    }
                    catch (Exception)
                    {

                    }

                }
            }
            else // hodnota byla null, tedy veličinu vymazáváme z monitoringu
            {
                if (aktVelicinyHodnoty.ContainsKey(trojice.Item1))
                {
                    try
                    {
                        DisplayData("Od " + trojice.Item3 + ": " + trojice.Item1 + ": smazat" /*+ " " + DateTime.Now */);
                    }
                    catch (Exception)
                    {

                    }
                    // NEW: mazani ma dovolenu jen zarizeni, ktere ma danou hodnotu jako vystupni a oznacenou
                    if (aktScenar.VystupniVelicinyProZarizeni[trojice.Item3].Contains(trojice.Item1))
                    {
                        aktVelicinyHodnoty.Remove(trojice.Item1);
                        try
                        {
                            DisplayMonitorData();
                        }
                        catch (Exception)
                        {

                        }
                    }
                    else
                    {
                        try
                        {
                            DisplayData("není ve výstupních veličinách, neprovedeno");
                        }
                        catch (Exception)
                        {

                        }
                    }
                }
            }
        }

        /// <summary>
        /// Metoda pro vložení do fronty veličin pro ukládání na globální monitoring
        /// </summary>
        /// <param name="task">parametrem je trojice (Tuple): Veličina, stringová hodnota a Zařízení</param>
        public static void vlozDoFrontyVelicin(Tuple<Velicina, string, Zarizeni> task)
        {
            FrontaVelicin.EnqueueTask(task);
        }

        static TaskQueue<Tuple<Udalost, string, Zarizeni>> FrontaUdalosti;
        static Tuple<Udalost, string, Zarizeni> trojiceUdal;
        /// <summary>
        /// Rozesílá Události Zařízením, které je mají nastaveny na vstupu. Je volaná z fronty událostí, odkud vybírá trojice (Tuple) Událost,
        /// hodnota (ta může být prázdná) a Zařízení. Volitelně zapisuje do logu. Pokud je Událost v aktuálním scénáři u zařazeného Zařízení v
        /// jeho vstupních událostech, tak mu ji předá k provedení. Rozešle jen, když zdrojové zařízení má danou událost ve scénáři na výstupu.
        /// </summary>
        /// <param name="task">musí být trojice (Tuple): Událost, stringová hodnota (ta může být prázdná) a Zařízení (odkud)</param>
        public void RozesliUdalost(object task)
        {
            trojiceUdal = (Tuple<Udalost, string, Zarizeni>)task;
            DisplayData("Událost od " + trojiceUdal.Item3 + ": " + trojiceUdal.Item1.Id + ": " + trojiceUdal.Item2 /*+ " " + DateTime.Now */);
            if (aktScenar != null)
            {
                if (aktScenar.VystupniUdalostiProZarizeni.ContainsKey(trojiceUdal.Item3) && aktScenar.VystupniUdalostiProZarizeni[trojiceUdal.Item3].Contains(trojiceUdal.Item1))
                {
                    foreach (var item in aktivniZarizeni)
                    {
                        if (aktScenar != null && aktScenar.VstupniUdalostiProZarizeni[item].Contains(trojiceUdal.Item1))
                        {
                            item.ProvedUdalost(trojiceUdal.Item1, trojiceUdal.Item2);
                        }
                    }
                }
                else
                {
                    DisplayData("není ve výstupních událostech, zahozeno");
                }
            }
        }

        /// <summary>
        /// Metoda pro vložení do fronty událostí pro jejich rozesílání
        /// </summary>
        /// <param name="task">parametrem je trojice (Tuple): Událost, stringová hodnota (ta může být prázdná) a Zařízení (odkud)</param>
        public static void vlozDoFrontyUdalosti(Tuple<Udalost, string, Zarizeni> task)
        {
            FrontaUdalosti.EnqueueTask(task);
        }

        /// <summary>
        /// Metoda pro pomocné zobrazení (log)
        /// </summary>
        /// <param name="msg">zpráva k zobrazení</param>
        private void DisplayData(string msg)
        {
            if (ukazovat)
            {
                textBox1.Invoke(new EventHandler(delegate
                {
                    textBox1.AppendText(msg + "\r\n");
                }));
            }
        }

        /// <summary>
        /// Metoda pro zobrazení monitorovaných veličin a hodnot do gridu
        /// </summary>
        private void DisplayMonitorData()
        {
            monitorDataGridView.Invoke(new EventHandler(delegate
            {
                monitorDataGridView.DataSource = (from d in aktVelicinyHodnoty select new { d.Key, d.Value }).ToList();
            }));
        }

        // provozní informace konkrétního měření
        static Scenar aktScenar = null;
        static Dictionary<Velicina, string> aktVelicinyHodnoty = new Dictionary<Velicina, string>();
        static bool ukazovat = false;
        static bool ukazovatOknaZarizeni = false;
        bool merime = false;
        List<Zarizeni> aktivniZarizeni = new List<Zarizeni>();

        /// <summary>
        /// Vrátí aktuální acénář
        /// </summary>
        /// <returns>aktuální scénář</returns>
        public static Scenar vratAktScenar()
        {
            return aktScenar;
        }

        /// <summary>
        /// Vrátí slovník aktuálních hodnot naměřených veličin
        /// </summary>
        /// <returns>slovník aktuálně naměřených hodnot veličin</returns>
        public static Dictionary<Velicina, string> vratAktVelicinyHodnoty()
        {
            return aktVelicinyHodnoty;
        }

        /// <summary>
        /// Vrátí příznak, jestli máme zapisovat do logu
        /// </summary>
        /// <returns>příznak o zápisu do logu</returns>
        public static bool vratUkazovat()
        {
            return ukazovat;
        }

        /// <summary>
        /// Vrátí příznak, jestli máme zobrazit okna zařízení
        /// </summary>
        /// <returns>příznak o zobrazení oken zařízení</returns>
        public static bool vratUkazovatOknaZarizeni()
        {
            return ukazovatOknaZarizeni;
        }

        /// <summary>
        /// start a stop konkrétního měření
        /// </summary>
        private void startStopButton_Click(object sender, EventArgs e)
        {
            if (!merime) // úkony pro start měření
            {
                aktScenar = (Scenar)vyberScenarComboBox.SelectedItem;
                if (aktScenar != null)
                {
                    ukazovat = ukazLogCheckBox.Checked;
                    ukazovatOknaZarizeni = ukazOknaCheckBox.Checked;

                    FrontaVelicin = new TaskQueue<Tuple<Velicina, string, Zarizeni>>(this);
                    FrontaUdalosti = new TaskQueue<Tuple<Udalost, string, Zarizeni>>(this);

                    aktivniZarizeni.Clear();
                    foreach (Zarizeni zarizeni in aktScenar.ZarazenaZarizeni)
                    {
                        try
                        {
                            zarizeni.InicializujKomunikaci();
                            aktivniZarizeni.Add(zarizeni);
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show("Došlo k chybě (" + ex.Message + ") při inicializaci komunikační komponenty zařízení " + zarizeni);
                        }
                    }
                    
                    foreach (Zarizeni zarizeni in aktivniZarizeni)
                    {
                        try
                        {
                            zarizeni.IncializujSe();
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show("Došlo k chybě (" + ex.Message + ") při inicializaci zařízení " + zarizeni);
                        }
                    }

                    textBox1.Text = "";
                    aktVelicinyHodnoty.Clear();
                    monitorDataGridView.DataSource = (from d in aktVelicinyHodnoty select new { d.Key, d.Value }).ToList();
                    monitorDataGridView.Columns.GetFirstColumn(DataGridViewElementStates.None).HeaderText = "Veličina";
                    monitorDataGridView.Columns.GetLastColumn(DataGridViewElementStates.None, DataGridViewElementStates.None).HeaderText = "Hodnota";
                    int width1 = monitorDataGridView.Width / 2;
                    monitorDataGridView.Columns.GetFirstColumn(DataGridViewElementStates.None).Width = width1 - 2;
                    monitorDataGridView.Columns.GetLastColumn(DataGridViewElementStates.None, DataGridViewElementStates.None).Width = monitorDataGridView.Width - width1 - 2;

                    ukazLogCheckBox.Enabled = false;
                    ukazOknaCheckBox.Enabled = false;
                    konfiguraceButton.Enabled = false;
                    vyberScenarComboBox.Enabled = false;
                    udalGroupBox.Enabled = true;

                    startStopButton.Text = "Stop";
                    merime = true;
                }
            }
            else         // úkony pro konec měření
            {
                foreach (Zarizeni zarizeni in aktivniZarizeni)
                {
                    try
                    {
                        zarizeni.DeInicializujSe();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Došlo k chybě (" + ex.Message + ") při deinicializaci zařízení " + zarizeni);
                    }
                }

                foreach (Zarizeni zarizeni in aktivniZarizeni)
                {
                    try
                    {
                        zarizeni.DeInicializujKomunikaci();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Došlo k chybě (" + ex.Message + ") při deinicializaci komunikační komponenty zařízení " + zarizeni);
                    }
                }
                
                FrontaVelicin.Dispose();
                FrontaUdalosti.Dispose();

                ukazLogCheckBox.Enabled = true;
                ukazOknaCheckBox.Enabled = true;
                konfiguraceButton.Enabled = true;
                vyberScenarComboBox.Enabled = true;
                udalGroupBox.Enabled = false;

                aktivniZarizeni.Clear();
                aktScenar = null;
                startStopButton.Text = "Start";
                merime = false;
            }
        }

        /// <summary>
        /// Vyvolání konfiguračného okna, po provedení změn se serializují seznamy
        /// </summary>
        private void konfiguraceButton_Click(object sender, EventArgs e)
        {
            KonfiguraceForm konfig = new KonfiguraceForm(vyberScenarComboBox.SelectedIndex);
            konfig.ShowDialog();
            vyberScenarComboBox.DataSource = null;
            vyberScenarComboBox.DataSource = Scenar.SeznamScenaru;
            udalostRadioButton_CheckedChanged(null, null);
            velicinaRadioButton_CheckedChanged(null, null);
            vyberZarizeniComboBox.DataSource = null;
            vyberZarizeniComboBox.DataSource = Zarizeni.SeznamZarizeni;

            using (XmlWriter w = XmlWriter.Create(jmenoKonfigSouboru, settings))
                dataSerializer.WriteObject(w, objektSeznamu);
        }

        /// <summary>
        /// Objekt sdružující serializované seznamy
        /// </summary>
        [DataContract]
        public class ObjektKSerializaci
        {
            [DataMember (Order = 0)] public List<Velicina> SeznamVelicin;
            [DataMember (Order = 1)] public List<Udalost> SeznamUdalosti;
            [DataMember (Order = 2)] public List<Zarizeni> SeznamZarizeni;
            [DataMember (Order = 3)] public List<Scenar> SeznamScenaru;

            public ObjektKSerializaci(List<Velicina> sezVel, List<Udalost> sezUdal, List<Zarizeni> sezZar, List<Scenar> sezScen)
            {
                this.SeznamVelicin = sezVel;
                this.SeznamUdalosti = sezUdal;
                this.SeznamZarizeni = sezZar;
                this.SeznamScenaru = sezScen;
            }
        }

        /// <summary>
        /// Při zavírání okna se ukončí měření (pokud ještě probíhá) a serializují se seznamy
        /// </summary>
        private void MereniForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (merime)
            {
                startStopButton_Click(null, null);
            }
            
            using (XmlWriter w = XmlWriter.Create(jmenoKonfigSouboru, settings))
                dataSerializer.WriteObject(w, objektSeznamu);
        }

        /// <summary>
        /// Vyvolá vložení vybrané veličiny nebo události na monitoring
        /// </summary>
        private void vyvolatButton_Click(object sender, EventArgs e)
        {
            if (udalostRadioButton.Checked)
            {
                vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>((Udalost)vyberUdalostVelicinuComboBox.SelectedItem, hodnTextBox.Text, (Zarizeni)vyberZarizeniComboBox.SelectedItem));
            }
            if (velicinaRadioButton.Checked)
            {
                vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>((Velicina)vyberUdalostVelicinuComboBox.SelectedItem, hodnTextBox.Text, (Zarizeni)vyberZarizeniComboBox.SelectedItem));
            }
        }

        private void udalostRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (udalostRadioButton.Checked)
            {
                vyberUdalostVelicinuComboBox.DataSource = null;
                vyberUdalostVelicinuComboBox.DataSource = Udalost.SeznamUdalosti;
            }
        }

        private void velicinaRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (velicinaRadioButton.Checked)
            {
                vyberUdalostVelicinuComboBox.DataSource = null;
                vyberUdalostVelicinuComboBox.DataSource = Velicina.SeznamVelicin;
            }
        }

        /// <summary>
        /// Znovu připojí vybrané zařízení (pokud je zařazeno do aktuálního scénáře)
        /// </summary>
        private void reConnectButton_Click(object sender, EventArgs e)
        {
            if (aktivniZarizeni.Contains(vyberZarizeniComboBox.SelectedItem))
            {
                try
                {
                    ((Zarizeni)vyberZarizeniComboBox.SelectedItem).DeInicializujSe();
                }
                catch (Exception)
                {
                    
                }

                try
                {
                    ((Zarizeni)vyberZarizeniComboBox.SelectedItem).DeInicializujKomunikaci();
                }
                catch (Exception)
                {
                    
                }

                aktivniZarizeni.Remove((Zarizeni)vyberZarizeniComboBox.SelectedItem);
            }

            if (aktScenar.ZarazenaZarizeni.Contains(vyberZarizeniComboBox.SelectedItem))
            {
                try
                {
                    ((Zarizeni)vyberZarizeniComboBox.SelectedItem).InicializujKomunikaci();
                    aktivniZarizeni.Add((Zarizeni)vyberZarizeniComboBox.SelectedItem);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Došlo k chybě (" + ex.Message + ") při inicializaci komunikační komponenty vybraného zařízení.");
                }

                if (aktivniZarizeni.Contains(vyberZarizeniComboBox.SelectedItem))
                {
                    try
                    {
                        ((Zarizeni)vyberZarizeniComboBox.SelectedItem).IncializujSe();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Došlo k chybě (" + ex.Message + ") při inicializaci vybraného zařízení.");
                    }
                }
            }
            else
            {
                MessageBox.Show("Vybrané zařízení není zařazeno v aktuálním scénáři, proto nemůže být připojeno.");
            }
        }

        /// <summary>
        /// Nastaví globální seznamy veličin, událostí, zařízení a scénářů dle modelového zapojení
        /// </summary>
        void NastavModeloveZapojeni()
        {
            // --- Veličiny
            Velicina.SeznamVelicin = new List<Velicina>();

            Velicina.SeznamVelicin.Add(new Velicina("Požadovaný výkon Auto", "W_set_auto", "W_set_auto", "W", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Požadovaný výkon Manual", "W_set_manual", "W_set_manual", "W", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální výkon", "W_act", "W_act", "W", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální otáčky", "R_act", "R_act", "rpm", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální tepová frekvence", "HR_act", "HR_act", "/ min", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální saturace", "SpO2_act", "SpO2_act", "%", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Systolický tlak", "BPS_act", "BPS_act", "mmHg", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Diastolický tlak", "DBP_act", "DBP_act", "mmHg", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Požadovaná rychlost Auto", "Speed_set_auto", "Speed_set_auto", "km/h", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Požadovaná rychlost Manual", "Speed_set_manual", "Speed_set_manual", "km/h", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální rychlost", "Speed_act", "Speed_act", "km/h", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Požadovaný sklon Auto", "Slope_set_auto", "Slope_set_auto", "%", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Požadovaný sklon Manual", "Slope_set_manual", "Slope_set_manual", "%", ""));
            Velicina.SeznamVelicin.Add(new Velicina("Aktuální sklon", "Slope_act", "Slope_act", "%", ""));


            // --- Události
            Udalost.SeznamUdalosti = new List<Udalost>();

            Udalost.SeznamUdalosti.Add(new Udalost("Init", "init", "", ""));
            Udalost.SeznamUdalosti.Add(new Udalost("Start", "start", "", ""));
            Udalost.SeznamUdalosti.Add(new Udalost("StartWithBP", "start_with_BP", "", ""));
            Udalost.SeznamUdalosti.Add(new Udalost("Stop", "stop", "", ""));
            Udalost.SeznamUdalosti.Add(new Udalost("Remark", "remark", "", ""));
            Udalost.SeznamUdalosti.Add(new Udalost("BP measurement", "BPm", "", ""));


            // --- Zařízení
            Zarizeni.SeznamZarizeni = new List<Zarizeni>();

            List<Velicina> listVs = new List<Velicina>();
            listVs.Add(Velicina.SeznamVelicin[0]); // kolo-vstup-pozadovany vykon Auto
            listVs.Add(Velicina.SeznamVelicin[1]); // kolo-vstup-pozadovany vykon Manual
            List<Velicina> listVy = new List<Velicina>();
            listVy.Add(Velicina.SeznamVelicin[2]); // kolo-vystup-aktualni vykon
            listVy.Add(Velicina.SeznamVelicin[3]); // kolo-vystup-aktualni otacky
            List<Udalost> listVsUdal = new List<Udalost>();
            listVsUdal.Add(Udalost.SeznamUdalosti[1]); // kolo-vstup-událost Start
            listVsUdal.Add(Udalost.SeznamUdalosti[2]); // kolo-vstup-událost Stop
            List<Udalost> listVyUdal = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("Kolo", "", "Ergoline_Slave", listVs, listVy, listVsUdal, listVyUdal, "COM28"));

            List<Velicina> listVs2 = new List<Velicina>();
            listVs2.Add(Velicina.SeznamVelicin[2]); // Oxycon-vstup-aktualni vykon
            listVs2.Add(Velicina.SeznamVelicin[3]); // Oxycon-vstup-aktualni otacky
            listVs2.Add(Velicina.SeznamVelicin[4]); // Oxycon-vstup-aktualni tepova frekvence
            listVs2.Add(Velicina.SeznamVelicin[5]); // Oxycon-vstup-aktualni saturace
            List<Velicina> listVy2 = new List<Velicina>();
            listVy2.Add(Velicina.SeznamVelicin[0]); // Oxycon-vystup-pozadovany vykon Auto
            List<Udalost> listVsUdal2 = new List<Udalost>();
            List<Udalost> listVyUdal2 = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("Oxycon", "", "Ergoline_Master", listVs2, listVy2, listVsUdal2, listVyUdal2, "COM2"));

            List<Velicina> listVs3 = new List<Velicina>();
            listVs3.Add(Velicina.SeznamVelicin[2]); // EKG-vstup-aktualni vykon
            listVs3.Add(Velicina.SeznamVelicin[3]); // EKG-vstup-aktualni otacky
            List<Velicina> listVy3 = new List<Velicina>();
            listVy3.Add(Velicina.SeznamVelicin[0]); // EKG-vystup-pozadovany vykon Auto
            listVy3.Add(Velicina.SeznamVelicin[4]); // EKG-vystup-aktualni tepova frekvence
            List<Udalost> listVsUdal3 = new List<Udalost>();
            List<Udalost> listVyUdal3 = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("EKG", "", "Ergoline_Master", listVs3, listVy3, listVsUdal3, listVyUdal3, "COM24"));

            List<Velicina> listVs4 = new List<Velicina>(); // Pulzak-vstup - je prazdny
            List<Velicina> listVy4 = new List<Velicina>();
            listVy4.Add(Velicina.SeznamVelicin[4]); // Pulzak-vystup-aktualni tepova frekvence
            listVy4.Add(Velicina.SeznamVelicin[5]); // Pulzak-vystup-aktualni saturace
            List<Udalost> listVsUdal4 = new List<Udalost>();
            List<Udalost> listVyUdal4 = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("Pulsní oxymetr", "", "Nonin_Slave", listVs4, listVy4, listVsUdal4, listVyUdal4, "COM10"));

            List<Velicina> listVs5 = new List<Velicina>();
            listVs5.Add(Velicina.SeznamVelicin[8]); // Behatko-vstup-pozadovana rychlost Auto
            listVs5.Add(Velicina.SeznamVelicin[9]); // Behatko-vstup-pozadovana rychlost Manual
            listVs5.Add(Velicina.SeznamVelicin[11]); // Behatko-vstup-pozadovany sklon Auto
            listVs5.Add(Velicina.SeznamVelicin[12]); // Behatko-vstup-pozadovany sklon Manual
            List<Velicina> listVy5 = new List<Velicina>();
            listVy5.Add(Velicina.SeznamVelicin[10]); // Behatko-vystup-aktualni rychlost
            listVy5.Add(Velicina.SeznamVelicin[13]); // Behatko-vystup-aktualni sklon
            List<Udalost> listVsUdal5 = new List<Udalost>();
            listVsUdal5.Add(Udalost.SeznamUdalosti[0]); // Behatko-vstup-událost Init
            listVsUdal5.Add(Udalost.SeznamUdalosti[1]); // Behatko-vstup-událost Start
            listVsUdal5.Add(Udalost.SeznamUdalosti[2]); // Behatko-vstup-událost Stop
            List<Udalost> listVyUdal5 = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("Běhátko", "", "T2000_Slave", listVs5, listVy5, listVsUdal5, listVyUdal5, "COM5"));

            List<Velicina> listVs6 = new List<Velicina>(); // BTL EKG-vstup - je prazdny
            List<Velicina> listVy6 = new List<Velicina>();
            listVy6.Add(Velicina.SeznamVelicin[4]); // BTL EKG-vystup-aktualni tepova frekvence
            List<Udalost> listVsUdal6 = new List<Udalost>();
            List<Udalost> listVyUdal6 = new List<Udalost>();
            Zarizeni.SeznamZarizeni.Add(new Zarizeni("BTL EKG", "", "BtlEcg_Slave", listVs6, listVy6, listVsUdal6, listVyUdal6, "COM53"));
            

            // --- Scénáře
            Scenar.SeznamScenaru = new List<Scenar>();

            Scenar.SeznamScenaru.Add(new Scenar("Modelové zapojení", "model", "", new List<Zarizeni>(), new SlovnikZarizeniVeliciny(), new SlovnikZarizeniVeliciny(), new SlovnikZarizeniUdalosti(), new SlovnikZarizeniUdalosti()));

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[0]); // kolo
            List<Velicina> koloVstup = new List<Velicina>();
            koloVstup.Add(Velicina.SeznamVelicin[0]); // kolo-vstup-pozadovany vykon Auto
            // koloVstup.Add(Velicina.SeznamVelicin[1]); // kolo-vstup-pozadovany vykon Manual - pouze volitelně
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[0], koloVstup); // kolo-vstup
            List<Udalost> koloVstupUdal = new List<Udalost>();
            koloVstupUdal.Add(Udalost.SeznamUdalosti[1]); // kolo-vstup-událost Start
            koloVstupUdal.Add(Udalost.SeznamUdalosti[2]); // kolo-vstup-událost Stop
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[0], koloVstupUdal); // kolo-vstup-události
            List<Velicina> koloVystup = new List<Velicina>();
            koloVystup.Add(Velicina.SeznamVelicin[2]); // kolo-vystup-aktualni vykon
            koloVystup.Add(Velicina.SeznamVelicin[3]); // kolo-vystup-aktualni otacky
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[0], koloVystup); // kolo-vystup
            List<Udalost> koloVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[0], koloVystupUdal); // kolo-vystup-události

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[1]); // Oxycon
            List<Velicina> oxyconVstup = new List<Velicina>();
            oxyconVstup.Add(Velicina.SeznamVelicin[2]); // Oxycon-vstup-aktualni vykon
            oxyconVstup.Add(Velicina.SeznamVelicin[3]); // Oxycon-vstup-aktualni otacky
            oxyconVstup.Add(Velicina.SeznamVelicin[4]); // Oxycon-vstup-aktualni tepova frekvence
            oxyconVstup.Add(Velicina.SeznamVelicin[5]); // Oxycon-vstup-aktualni saturace
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[1], oxyconVstup); // Oxycon-vstup
            List<Udalost> oxyconVstupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[1], oxyconVstupUdal); // Oxycon-vstup-události
            List<Velicina> oxyconVystup = new List<Velicina>();
            oxyconVystup.Add(Velicina.SeznamVelicin[0]); // Oxycon-vystup-pozadovany vykon Auto
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[1], oxyconVystup); // Oxycon-vystup
            List<Udalost> oxyconVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[1], oxyconVystupUdal); // Oxycon-vstup-události

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[2]); // EKG
            List<Velicina> ekgVstup = new List<Velicina>();
            ekgVstup.Add(Velicina.SeznamVelicin[2]); // EKG-vstup-aktualni vykon
            ekgVstup.Add(Velicina.SeznamVelicin[3]); // EKG-vstup-aktualni otacky
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[2], ekgVstup); // EKG-vstup
            List<Udalost> ekgVstupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[2], ekgVstupUdal); // EKG-vstup-události
            List<Velicina> ekgVystup = new List<Velicina>();
            ekgVystup.Add(Velicina.SeznamVelicin[0]); // EKG-vystup-pozadovany vykon Auto
            ekgVystup.Add(Velicina.SeznamVelicin[4]); // EKG-vystup-aktualni tepova frekvence
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[2], ekgVystup); // EKG-vystup
            List<Udalost> ekgVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[2], ekgVystupUdal); // EKG-vystup-události

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[3]); // Pulzak
            List<Velicina> pulzakVstup = new List<Velicina>();
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[3], pulzakVstup); // Pulzak-vstup - je prazdny
            List<Udalost> pulzakVstupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[3], pulzakVstupUdal); // Pulzak-vstup-události
            List<Velicina> pulzakVystup = new List<Velicina>();
            // pulzakVystup.Add(Velicina.SeznamVelicin[4]); // Pulzak-vystup-aktualni tepova frekvence-zahazujeme
            pulzakVystup.Add(Velicina.SeznamVelicin[5]); // Pulzak-vystup-aktualni saturace
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[3], pulzakVystup); // Pulzak-vystup
            List<Udalost> pulzakVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[3], pulzakVystupUdal); // Pulzak-vystup-události

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[4]); // Behatko
            List<Velicina> behatkoVstup = new List<Velicina>();
            behatkoVstup.Add(Velicina.SeznamVelicin[8]); // Behatko-vstup-pozadovana rychlost Auto
            // behatkoVstup.Add(Velicina.SeznamVelicin[9]); // Behatko-vstup-pozadovana rychlost Manual - pouze volitelně
            behatkoVstup.Add(Velicina.SeznamVelicin[11]); // Behatko-vstup-pozadovany sklon Auto
            // behatkoVstup.Add(Velicina.SeznamVelicin[12]); // Behatko-vstup-pozadovany sklon Manual - pouze volitelně
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[4], behatkoVstup); // Behatko-vstup
            List<Udalost> behatkoVstupUdal = new List<Udalost>();
            behatkoVstupUdal.Add(Udalost.SeznamUdalosti[0]); // Behatko-vstup-událost Init
            behatkoVstupUdal.Add(Udalost.SeznamUdalosti[1]); // Behatko-vstup-událost Start
            behatkoVstupUdal.Add(Udalost.SeznamUdalosti[2]); // Behatko-vstup-událost Stop
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[4], behatkoVstupUdal); // Behatko-vstup-události
            List<Velicina> behatkoVystup = new List<Velicina>();
            behatkoVystup.Add(Velicina.SeznamVelicin[10]); // Behatko-vystup-aktualni rychlost
            behatkoVystup.Add(Velicina.SeznamVelicin[13]); // Behatko-vystup-aktualni sklon
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[4], behatkoVystup); // Behatko-vystup
            List<Udalost> behatkoVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[4], behatkoVystupUdal); // Behatko-vystup-události

            Scenar.SeznamScenaru[0].ZarazenaZarizeni.Add(Zarizeni.SeznamZarizeni[5]); // BTL EKG
            List<Velicina> btlEcgVstup = new List<Velicina>();
            Scenar.SeznamScenaru[0].VstupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[5], btlEcgVstup); // BTL EKG-vstup - je prazdny
            List<Udalost> btlEcgVstupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VstupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[5], btlEcgVstupUdal); // BTL EKG-vstup-události
            List<Velicina> btlEcgVystup = new List<Velicina>();
            btlEcgVystup.Add(Velicina.SeznamVelicin[4]); // BTL EKG-vystup-aktualni tepova frekvence
            Scenar.SeznamScenaru[0].VystupniVelicinyProZarizeni.Add(Zarizeni.SeznamZarizeni[5], btlEcgVystup); // BTL EKG-vystup
            List<Udalost> btlEcgVystupUdal = new List<Udalost>();
            Scenar.SeznamScenaru[0].VystupniUdalostiProZarizeni.Add(Zarizeni.SeznamZarizeni[5], btlEcgVystupUdal); // BTL EKG-vystup-události


            // --- Objekt k serializaci
            objektSeznamu = new ObjektKSerializaci(Velicina.SeznamVelicin, Udalost.SeznamUdalosti, Zarizeni.SeznamZarizeni, Scenar.SeznamScenaru);
        }
    }
}
