﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class Nonin_Slave : Protokol
    {
        // Regexy pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^SpO2=( |\d)\d{2} HR=( |\d)\d{2} $");
        Regex myRegex2 = new Regex(@"^SpO2=--- HR=--- $");

        // Vazba mezi "příkazy" a veličinami pro rozpoznávání příkazů (Response from Nonin)
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "1", "SpO2_act" }, { "2", "HR_act" } };
        
        public override string VypisJmeno()
        {
            return "Nonin_Slave";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r\n";
            vlastnosti.BaudRate = 9600;
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.Two;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            // Timer se nepoužívá
        }

        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            // nepoužívá se
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            return "nepodporováno";
        }

        Velicina velicina;
        string ret;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexem: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                // ukládání hodnot
                ret = "";
                velicina = Velicina.vyhledejVelicinu(prikazyId["1"]); // to je vyhledání SpO2_act
                // pokud rozpozná veličinu (saturace), vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(5, 3), odkud));
                }
                else
                {
                    ret += "nerozpoznáno Id saturace ";
                }

                velicina = Velicina.vyhledejVelicinu(prikazyId["2"]); // to je vyhledání HR_act
                // pokud rozpozná veličinu (TF), vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(12, 3), odkud));
                }
                else
                {
                    ret += "nerozpoznáno Id tep.frekvence";
                }
                return ret;
            }
            else if (myRegex2.IsMatch(prikaz))
            {
                // jedná se o pomlčky, tedy odesíláme hodnotu null pro obě veličiny
                ret = "";
                velicina = Velicina.vyhledejVelicinu(prikazyId["1"]); // to je vyhledání SpO2_act
                // pokud rozpozná veličinu (saturace), vloží ji s hodnotou null do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, null, odkud));
                }
                else
                {
                    ret += "nerozpoznáno Id saturace ";
                }

                velicina = Velicina.vyhledejVelicinu(prikazyId["2"]); // to je vyhledání HR_act
                // pokud rozpozná veličinu (TF), vloží ji s hodnotou null do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, null, odkud));
                }
                else
                {
                    ret += "nerozpoznáno Id tep.frekvence";
                }
                return ret;
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return null;
        }
    }
}
