﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CPXDI_model
{
    public abstract class Protokol
    {
        /// <summary>
        /// Vypíše jméno protokolu
        /// </summary>
        /// <returns>Jméno</returns>
        public abstract string VypisJmeno();

        /// <summary>
        /// Nastaví parametry komunikační komponenty zařízení (např. parametry jejího portu) dle svých potřeb
        /// </summary>
        /// <param name="vlastnosti">Nastavované vlastnosti</param>
        public abstract void NastavKomunikaci(VlastnostiKomunikace vlastnosti);

        /// <summary>
        /// Nastaví parametry zařízení (kromě vlastností Komunikace) dle svých potřeb
        /// </summary>
        /// <param name="zarizeni">Nastavované zařízení</param>
        public abstract void NastavZarizeni(Zarizeni zarizeni);

        /// <summary>
        /// Obslouží zařízení provedením potřebných periodických úkonů (např. dotazy) - volá se timerem ze zařízení (pokud je zapnut)
        /// </summary>
        /// <param name="zarizeni">Obsluhované zařízení</param>
        public abstract void ObsluzZarizeni(Zarizeni zarizeni);

        /// <summary>
        /// Obslouží danému zařízení reakci na nastalou událost
        /// </summary>
        /// <param name="udalost">Nastalá událost</param>
        /// <param name="hodnota">poznámka k události (může být prázdná)</param>
        /// <param name="komu">Zařízení, kterému se událost obsluhuje</param>
        /// <returns>Zde může protokol vrátit do zařízení přiměřenou zprávu o úspěšnosti rozpoznání Události, kterou zařízení zobrazí ve svém
        /// logu (hlavně pokud není Událost podporována). Pokud obsloužení proběhlo v pořádku, očekává se vrácení prázdného řetězce.</returns>
        public abstract string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu);

        /// <summary>
        /// Dostane příkaz od zařízení a zpracuje ho dle svých potřeb (např. uloží na monitoring nebo pošle odpověď)
        /// </summary>
        /// <param name="prikaz">Došlý příkaz</param>
        /// <param name="odkud">Zařízení, od kterého příkaz přišel</param>
        /// <returns>Zde může protokol vrátit do zařízení přiměřenou zprávu o úspěšnosti rozpoznání příkazu, kterou zařízení zobrazí ve svém
        /// logu (hlavně pokud nebyl příkaz rozpoznán). Pokud rozpoznání proběhlo v pořádku, očekává se vrácení prázdného řetězce.</returns>
        public abstract string ZpracujPrikaz(string prikaz, Zarizeni odkud);

        /// <summary>
        /// Zde může protokol vrátit předem připravený slovník příkazů, které podporuje a bude je tedy možno ručně (či automaticky) posílat
        /// přímo do zařízení (budou zobrazeny v ovládacím okně daného zařízení). Jedná se především o příkazy ovládání (start, stop, apod.).
        /// Ve slovníku je klíčem název či popis příkazu a hodnotou konkrétní zápis daného příkazu, který se pošle do zařízení.
        /// </summary>
        /// <returns>Slovník: Klíč: název či popis příkazu; Hodnota: přesné znění příkazu pro odeslání do zařízení</returns>
        public abstract Dictionary<string, string> DefinujPrikazyOvladani();

        public override string ToString()
        {
            try
            {
                return VypisJmeno();
            }
            catch (Exception)
            {
                return base.ToString();
            }
        }
    }
}
