﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace CPXDI_model
{
    [CollectionDataContract (ItemName = "PolozkaSlovniku", KeyName = "Klic-Zarizeni", ValueName = "Hodnota-SeznamVelicin")]
    public class SlovnikZarizeniVeliciny : Dictionary<Zarizeni, List<Velicina>> { }

    [CollectionDataContract (ItemName = "PolozkaSlovniku", KeyName = "Klic-Zarizeni", ValueName = "Hodnota-SeznamUdalosti")]
    public class SlovnikZarizeniUdalosti : Dictionary<Zarizeni, List<Udalost>> { }
    
    [DataContract]
    public class Scenar
    {
        [DataMember (Order = 0)] public string Nazev;
        [DataMember (Order = 1)] public string Id;
        [DataMember (Order = 2)] public string Popis;
        [DataMember (Order = 3)] public List<Zarizeni> ZarazenaZarizeni;
        [DataMember (Order = 4)] public SlovnikZarizeniVeliciny VstupniVelicinyProZarizeni;
        [DataMember (Order = 5)] public SlovnikZarizeniVeliciny VystupniVelicinyProZarizeni;
        [DataMember (Order = 6)] public SlovnikZarizeniUdalosti VstupniUdalostiProZarizeni;
        [DataMember (Order = 7)] public SlovnikZarizeniUdalosti VystupniUdalostiProZarizeni;

        /// <summary>
        /// Konstruktor nového scénáře
        /// </summary>
        /// <param name="nazev">Název</param>
        /// <param name="popis">Popis</param>
        /// <param name="zarZ">Zařazená zařízení</param>
        /// <param name="vstupV">Slovník - pro každé zařízení seznam jeho vstupních veličin v daném scénáři</param>
        /// <param name="vystupV">Slovník - pro každé zařízení seznam jeho výstupních veličin v daném scénáři</param>
        /// <param name="vstupUd">Slovník - pro každé zařízení seznam jeho vstupních událostí v daném scénáři</param>
        /// <param name="vystupUd">Slovník - pro každé zařízení seznam jeho výstupních událostí v daném scénáři</param>
        public Scenar(string nazev, string id, string popis, List<Zarizeni> zarZ, SlovnikZarizeniVeliciny vstupV, SlovnikZarizeniVeliciny vystupV, SlovnikZarizeniUdalosti vstupUd, SlovnikZarizeniUdalosti vystupUd)
        {
            this.Nazev = nazev;
            this.Id = id;
            this.Popis = popis;
            this.ZarazenaZarizeni = zarZ;
            this.VstupniVelicinyProZarizeni = vstupV;
            this.VystupniVelicinyProZarizeni = vystupV;
            this.VstupniUdalostiProZarizeni = vstupUd;
            this.VystupniUdalostiProZarizeni = vystupUd;
        }

        public override string ToString()
        {
            return Nazev;
        }

        /// <summary>
        /// Globální seznam scénářů
        /// </summary>
        public static List<Scenar> SeznamScenaru = new List<Scenar>();
    }
}
