﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class T2000_Master : Protokol
    {
        // Regexy pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^(S|G)\d{4}$");
        Regex myRegex2 = new Regex(@"^W$");
        Regex myRegex3 = new Regex(@"^(C|T|V)$");
        Regex myRegex4 = new Regex(@"^(A|B|E|F|I|K|L|N|Q|U|Z|H|P|M|R|X)$");

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "S", "Speed_set_auto" }, { "G", "Slope_set_auto" } };

        // Vazba mezi veličinami a příkazy pro posílání hodnot
        Dictionary<string, string> idPrikazy = new Dictionary<string, string>() { { "Speed_act", "S" }, { "Slope_act", "G" } };
        
        public override string VypisJmeno()
        {
            return "T2000_Master";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.BaudRate = 9600;
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.One;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            // Timer se nepoužívá
        }

        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            // nepoužívá se
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            return "nepodporováno";
        }

        Velicina velicina;
        string hodnota;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexy: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz.Substring(0, 1)]);
                // pokud rozpozná veličinu, vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(1), odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else if (myRegex2.IsMatch(prikaz))
            {
                // What's up - poslání hodnot veličin ze slovníku idPrikazy (rychlost a sklon - příkazy S a G) + příkaz K
                foreach (var item in idPrikazy)
                {
                    velicina = Velicina.vyhledejVelicinu(item.Key);
                    if (velicina != null)
                    {
                        try
                        {
                            hodnota = MereniForm.vratAktVelicinyHodnoty()[velicina];
                        }
                        catch (Exception)
                        {
                            hodnota = "0000";
                        }
                        if (hodnota.Length != 4)
                        {
                            hodnota = "0000";
                        }
                        odkud.OdesliPrikaz(item.Value + hodnota);
                    }
                }
                odkud.OdesliPrikaz("K");
                return "";
            }
            else if (myRegex3.IsMatch(prikaz))
            {
                switch (prikaz)
                {
                    case "C": // č. verze SW
                        odkud.OdesliPrikaz("T2X5");
                        break;
                    case "T": // Mill type
                        odkud.OdesliPrikaz("K6");
                        break;
                    case "V": // Software version
                        odkud.OdesliPrikaz("008A");
                        break;
                    default:
                        break;
                }
                return "";
            }
            else if (myRegex4.IsMatch(prikaz))
            {
                // A, E, F, I, L, Q, U, Z, H, P, M, R, X nyní ignorujeme
                switch (prikaz)
                {
                    case "K": // K vytvoří událost Init
                        MereniForm.vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(Udalost.vyhledejUdalost("init"), "", odkud));
                        break;
                    case "B": // B vytvoří událost Start
                        MereniForm.vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(Udalost.vyhledejUdalost("start"), "", odkud));
                        break;
                    case "N": // N vytvoří událost Stop
                        MereniForm.vlozDoFrontyUdalosti(new Tuple<Udalost, string, Zarizeni>(Udalost.vyhledejUdalost("stop"), "", odkud));
                        break;
                    default:
                        break;
                }
                return "";
            }
            else
            {
                return "nerozpoznáno";
            }
        }

        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return null;
        }
    }
}
