﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace CPXDI_model
{
    public class T2000_Slave : Protokol
    {
        // Regex pro rozpoznávání příkazů
        Regex myRegex1 = new Regex(@"^(S|G)\d{4}$");

        // Vazba mezi příkazy a veličinami pro rozpoznávání příkazů
        Dictionary<string, string> prikazyId = new Dictionary<string, string>() { { "S", "Speed_act" }, { "G", "Slope_act" } };

        // vazba mezi veličinami a příkazy pro posílání hodnot
        Dictionary<string, string> idPrikazy = new Dictionary<string, string>() { { "Speed_set_auto", "S" }, { "Speed_set_manual", "S" },
            { "Slope_set_auto", "G" }, { "Slope_set_manual", "G" } };
        
        public override string VypisJmeno()
        {
            return "T2000_Slave";
        }

        public override void NastavKomunikaci(VlastnostiKomunikace vlastnosti)
        {
            vlastnosti.NewLine = "\r";
            vlastnosti.BaudRate = 9600;
            vlastnosti.DataBits = 8;
            vlastnosti.StopBits = StopBits.One;
            vlastnosti.Parity = Parity.None;
            vlastnosti.Handshake = Handshake.None;
        }

        public override void NastavZarizeni(Zarizeni zarizeni)
        {
            zarizeni.pouzivatTimer = true;
        }

        Scenar scenar;
        string hodnota;
        public override void ObsluzZarizeni(Zarizeni zarizeni)
        {
            // posílání požadovaných hodnot veličin (rychlost a sklon) - příkazy S a G
            scenar = MereniForm.vratAktScenar();
            if (scenar != null)
            {
                if (scenar.VstupniVelicinyProZarizeni != null)
                {
                    foreach (var item in scenar.VstupniVelicinyProZarizeni[zarizeni])
                    {
                        if (item.Id == "Speed_set_auto" || item.Id == "Speed_set_manual" || item.Id == "Slope_set_auto" || item.Id == "Slope_set_manual")
                        {
                            try
                            {
                                hodnota = MereniForm.vratAktVelicinyHodnoty()[item];
                            }
                            catch (Exception)
                            {
                                hodnota = "0000";
                            }
                            if (hodnota.Length != 4)
                            {
                                hodnota = "0000";
                            }
                            if (hodnota == "0000" && !zarizeni.posilatNeexistujiciHodnoty)
                            {
                                hodnota = "";
                            }
                            if (hodnota != "")
                            {
                                zarizeni.OdesliPrikaz(idPrikazy[item.Id] + hodnota);
                            }
                        }
                    }
                }
            }
            
            // posílání dotazu (What's up) - příkaz W
            zarizeni.OdesliPrikaz("W");
        }

        public override string ObsluzUdalost(Udalost udalost, string hodnota, Zarizeni komu)
        {
            switch (udalost.Id)
            {
                case "init":
                    komu.OdesliPrikaz("K");
                    return "";
                case "start":
                    komu.OdesliPrikaz("B");
                    return "";
                case "stop":
                    komu.OdesliPrikaz("N\rN\rN");
                    return "";
                default:
                    return "Nepodporovaná událost: " + udalost.Id;
            }
        }

        Velicina velicina;
        public override string ZpracujPrikaz(string prikaz, Zarizeni odkud)
        {
            // porovnání shody s Regexem: pokud nerozpozná, vrací zprávu pro zařízení, jinak vrací prázdný řetězec
            if (myRegex1.IsMatch(prikaz))
            {
                velicina = Velicina.vyhledejVelicinu(prikazyId[prikaz.Substring(0, 1)]);
                // pokud rozpozná veličinu, vloží ji s hodnotou do fronty pro globální monitoring
                if (velicina != null)
                {
                    MereniForm.vlozDoFrontyVelicin(new Tuple<Velicina, string, Zarizeni>(velicina, prikaz.Substring(1), odkud));
                    return "";
                }
                else
                {
                    return "nerozpoznáno Id veličiny";
                }
            }
            else
            {
                if (prikaz == "K")
                {
                    return "";
                }
                else
                {
                    return "nerozpoznáno";
                }
            }
        }

        Dictionary<string, string> slovnikPrikazu = new Dictionary<string, string>() {
            { "Start pásu B", "B" },
            { "Start pásu F", "F" },
            { "Normal Stop", "N" },
            { "Emergency Stop", "E\rE" },
            { "nastav km/h", "K" },
            { "nastav míle/h", "A" },
            { "Increment Speed", "I" },
            { "Lower Speed", "L" },
            { "Incremental up", "U" },
            { "č. verze SW", "C" },
            { "Software version", "V" },
            { "Mill type", "T" },
            { "do nothing", "Q" },
            { "No Command", "Z" },
            { "High Speed communication", "H" },
            { "Provide more info", "P" },
            { "Set the Accel/Decel Rate", "M007007" },
            { "soft reset", "R\rR" },
            { "Low speed character output", "X" }
        };
        public override Dictionary<string, string> DefinujPrikazyOvladani()
        {
            return slovnikPrikazu;
        }
    }
}
