﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace CPXDI_model
{
    /// <summary>
    /// Třída pro práci s frontou producent / konzument
    /// </summary>
    /// <typeparam name="T">typ musí být referencovatelný</typeparam>
    public class TaskQueue<T> : IDisposable where T : class
    {
        object locker = new object();
        Thread worker;
        Queue<T> taskQ = new Queue<T>();
        Zarizeni aktZarizeni;
        MereniForm aktForm;

        /// <summary>
        /// Konstruktor pro využití v zařízení
        /// </summary>
        /// <param name="zarizeni">Zařízení, které frontu používá</param>
        public TaskQueue(Zarizeni zarizeni)
        {
            aktZarizeni = zarizeni;
            
            (worker = new Thread(Consume)).Start();
        }

        /// <summary>
        /// Konstruktor pro využití ve Form1 (ukládání na monitoring)
        /// </summary>
        /// <param name="form">Form1, který frontu používá</param>
        public TaskQueue(MereniForm form)
        {
            aktForm = form;
            
            (worker = new Thread(Consume)).Start();
        }

        /// <summary>
        /// Zakončení vlákna
        /// </summary>
        public void Dispose()
        {
            EnqueueTask(null);
            worker.Abort();
            worker.Join();
        }

        /// <summary>
        /// Přidání úkolu do fronty
        /// </summary>
        /// <param name="task">Úkol daného typu</param>
        public void EnqueueTask(T task)
        {
            lock (locker)
            {
                /*if (task == null)
                {
                    taskQ.Clear();
                }*/
                taskQ.Enqueue(task);
                Monitor.PulseAll(locker);
            }
        }

        /// <summary>
        /// Zpracování úkolů
        /// </summary>
        void Consume()
        {
            while (true)
            {
                T task;
                lock (locker)
                {
                    while (taskQ.Count == 0) Monitor.Wait(locker);
                    task = taskQ.Dequeue();
                }
                if (task == null) return;   // konec

                Proved(task);
            }
        }

        /// <summary>
        /// Provedení činnosti podle typu úkolu
        /// </summary>
        /// <param name="task">Pro typ string se volá metoda ZpracujData aktuálního zařízení; Pro typ
        /// Tuple:Velicina,string,Zarizeni se volá metoda UlozData aktuálního Form1 (tj. uložení na monitoring) </param>
        void Proved(T task)
        {
            if (task.GetType() == typeof(string))
            {
                aktZarizeni.ZpracujData(task.ToString());
            }
            else if (task.GetType() == typeof(Tuple<Velicina,string,Zarizeni>))
            {
                aktForm.UlozData(task);
            }
            else if (task.GetType() == typeof(Tuple<Udalost,string,Zarizeni>))
            {
                aktForm.RozesliUdalost(task);
            }
        }
    }
}
