﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CPXDI_model
{
    public static class TvurceKomunikace
    {
        static Dictionary<string, string> slovnik = new Dictionary<string, string>() { { "Sériový port", "název COM portu (např. COM10)" },
            { "TCP klient", "ve tvaru IP_adresa:číslo_portu (např. 127.0.0.1:2000)" } };
        static char[] dvojtecka = new char[] { ':' };
        /// <summary>
        /// Slouží k získání slovníku názvů (a nápovědy) dostupných způsobů komunikace pro nastavení ve Správě zařízení
        /// </summary>
        /// <param name="komunikacniParametr">Komunikační parametr spravovaného zařízení</param>
        /// <param name="index">Index toho, který způsob komunikace ze slovníku odpovídá zadanému komunikačnímu parametru</param>
        /// <returns>Slovník dostupných způsobů komunikace: Klíčem je název, Hodnotou je nápověda, jak má vypadat odpovídající
        /// komunikační parametr</returns>
        public static Dictionary<string, string> VratSlovnik(string komunikacniParametr, out int index)
        {
            string[] pole = komunikacniParametr.Split(dvojtecka, 2, StringSplitOptions.RemoveEmptyEntries);
            if (pole.Length == 2)
            {
                index = 1;
            }
            else
            {
                index = 0;
            }
            return slovnik;
        }

        /// <summary>
        /// Slouží k získání nové instance Komunikace, kterou vrátí na základě přijatého Komunikačního parametru (např. název portu)
        /// </summary>
        /// <param name="komunikacniParametr">parametr pro komunikaci (např. název portu)</param>
        /// <param name="metoda">delegát na metodu, která zpracovává přijatá data</param>
        /// <returns>Vrátí instanci Komunikace na základě přijatého Komunikačního parametru</returns>
        public static Komunikace VratKomunikaci(string komunikacniParametr, Komunikace.Prijimaci metoda)
        {
            string[] pole = komunikacniParametr.Split(dvojtecka, 2, StringSplitOptions.RemoveEmptyEntries);
            if (pole.Length == 2)
            {
                int result;
                if (!int.TryParse(pole[1], out result))
                {
                    result = 2000;
                }
                return new TcpClientKomunikace(pole[0], result, metoda);
            }
            else
            {
                return new SerialPortKomunikace(komunikacniParametr, metoda);
            }
        }
    }

    public class VlastnostiKomunikace
    {
        // pro všechny druhy komunikace
        public string NewLine;

        // pro sériový port
        public int BaudRate;
        public int DataBits;
        public System.IO.Ports.StopBits StopBits;
        public System.IO.Ports.Parity Parity;
        public System.IO.Ports.Handshake Handshake;

        // pro TCP klienta
        public Encoding Encoding;
        public int BufferSize;
    }
}
