﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Timers;
using System.Runtime.Serialization;

namespace CPXDI_model
{
    [DataContract]
    public class Zarizeni
    {
        [DataMember (Order = 0)] public string Nazev;
        [DataMember (Order = 1)] public string Popis;
        [DataMember (Order = 2)] public string ProtokolNazev;
        [DataMember (Order = 3)] public List<Velicina> VstupniVeliciny;
        [DataMember (Order = 4)] public List<Velicina> VystupniVeliciny;
        [DataMember (Order = 5)] public List<Udalost> VstupniUdalosti;
        [DataMember (Order = 6)] public List<Udalost> VystupniUdalosti;
        [DataMember (Order = 7)] public string PortNazev;

        public Protokol VychoziProtokol;
        public Komunikace Komunikace;
        public Timer Casovac;
        public ZarizeniForm Okno;

        public TaskQueue<string> Fronta;
        public bool pouzivatTimer = false;
        public bool ukazovat = false;
        public bool posilatNeexistujiciHodnoty = false;

        [OnDeserialized]
        void poDeserializaci(StreamingContext context)
        {
            this.VychoziProtokol = TvurceProtokolu.VratProtokol(this.ProtokolNazev);
            this.Komunikace = TvurceKomunikace.VratKomunikaci(this.PortNazev, new Komunikace.Prijimaci(this.PrijmiDataKomunikace));
            this.Casovac = new Timer(1000); this.Casovac.Elapsed += new ElapsedEventHandler(casovac_Tick);
            this.Okno = new ZarizeniForm(this);
        }
        
        /// <summary>
        /// Konstruktor nového zařízení
        /// </summary>
        /// <param name="nazev">Název</param>
        /// <param name="popis">Popis</param>
        /// <param name="protokol">Název Protokolu</param>
        /// <param name="vstupniVeliciny">Vstupní veličiny</param>
        /// <param name="vystupniVeliciny">Výstupní veličiny</param>
        /// <param name="vstupniUdalosti">Vstupní události</param>
        /// <param name="vystupniUdalosti">Výstupní události</param>
        /// <param name="portNazev">Komunikační parametr (např. název portu)</param>
        public Zarizeni(string nazev, string popis, string protokol, List<Velicina> vstupniVeliciny, List<Velicina> vystupniVeliciny, List<Udalost> vstupniUdalosti, List<Udalost> vystupniUdalosti, string portNazev)
        {
            this.Nazev = nazev;
            this.Popis = popis;
            this.ProtokolNazev = protokol;
            this.VstupniVeliciny = vstupniVeliciny;
            this.VystupniVeliciny = vystupniVeliciny;
            this.VstupniUdalosti = vstupniUdalosti;
            this.VystupniUdalosti = vystupniUdalosti;
            this.PortNazev = portNazev;
            poDeserializaci(new StreamingContext());
        }

        public override string ToString()
        {
            return Nazev;
        }

        /// <summary>
        /// Globální seznam zařízení
        /// </summary>
        public static List<Zarizeni> SeznamZarizeni = new List<Zarizeni>();

        /// <summary>
        /// Inicializace komunikační komponenty (potřeba zavolat před inicializací zbytku zařízení)
        /// </summary>
        public void InicializujKomunikaci()
        {
            VlastnostiKomunikace vlastnosti = new VlastnostiKomunikace();
            VychoziProtokol.NastavKomunikaci(vlastnosti);
            Komunikace.InicializujSe(vlastnosti);
        }

        /// <summary>
        /// Inicializace zařízení (potřeba zavolat před začátkem měření, ale po inicializaci komunikace)
        /// </summary>
        public void IncializujSe()
        {
            VychoziProtokol.NastavZarizeni(this);
            Fronta = new TaskQueue<string>(this);
            Komunikace.Start();
            if (pouzivatTimer)
            {
                Casovac.Start();
            }
            
            if (Okno.IsDisposed)
            {
                Okno = new ZarizeniForm(this);
            }
            ukazovat = MereniForm.vratUkazovat();
            Okno.NastavTimerCheckBox(pouzivatTimer);
            Okno.NastavNeexHodCheckBox(posilatNeexistujiciHodnoty);
            if (MereniForm.vratUkazovatOknaZarizeni()) { Okno.Show(); }
        }

        /// <summary>
        /// DeInicializace zařízení (potřeba zavolat po ukončení měření, ale před deinicializací komunikace)
        /// </summary>
        public void DeInicializujSe()
        {
            pouzivatTimer = false;
            Casovac.Stop();
            Komunikace.Stop();
            Fronta.Dispose();
        }

        /// <summary>
        /// DeInicializace komunikační komponenty (potřeba zavolat po deinicializaci zbytku zařízení)
        /// </summary>
        public void DeInicializujKomunikaci()
        {
            Komunikace.DeInicializujSe();
        }

        /// <summary>
        /// Slouží jako EventHandler při tikání Timeru, volá metodu ObsluzZarizeni svého protokolu (pokud se má Timer používat)
        /// </summary>
        void casovac_Tick(object sender, EventArgs e)
        {
            if (pouzivatTimer)
            {
                // System.Threading.Thread.Sleep(5000); // zkouška pozastavení vlákna pro prodloužení běhu
                VychoziProtokol.ObsluzZarizeni(this);
            }
        }

        /// <summary>
        /// Slouží jako obsluha při čtení z komunikační komponenty: pokud je parametr ok true, tak vloží přijatý text (řádek) do fronty úkolů,
        /// jinak zobrazí přijatý text (chybovou hlášku) v logovacím okně zařízení
        /// </summary>
        /// <param name="text">přijatý řádek nebo chybová hláška</param>
        /// <param name="ok">signalizace úspěšného přečtení</param>
        internal void PrijmiDataKomunikace(string text, bool ok)
        {
            if (ok)
	        {
                Fronta.EnqueueTask(text);
	        }
            else
	        {
                if (ukazovat)
                {
                    try
                    {
                        Okno.DisplayData("Chyba při přijímání: (" + text + ")");
                    }
                    catch (Exception)
                    {

                    }
                }
	        }
        }

        string zprava;
        /// <summary>
        /// Dostane (z fronty úkolů) příkaz, zapíše ho na Log a předá svému protokolu ke zpracování
        /// </summary>
        /// <param name="data">obdržený příkaz</param>
        public void ZpracujData(string data)
        {
            if (ukazovat)
            {
                try
                {
                    Okno.DisplayData("OUT " + DateTime.Now + ": " + data);
                }
                catch (Exception)
                {

                }
                // System.Threading.Thread.Sleep(5000); // zkouška pozastavení vlákna pro prodloužení běhu
            }
            zprava = VychoziProtokol.ZpracujPrikaz(data, this);
            if (ukazovat && zprava != "")
            {
                try
                {
                    Okno.DisplayData(zprava);
                }
                catch (Exception)
                {

                }
            }
        }

        /// <summary>
        /// Provede svou vstupní událost (tj. zapíše ji na Log a předá svému protokolu k obsloužení)
        /// </summary>
        /// <param name="udalost">Vstupní událost</param>
        /// <param name="hodnota">poznámka k události (může být prázdná)</param>
        public void ProvedUdalost(Udalost udalost, string hodnota)
        {
            if (ukazovat)
            {
                try
                {
                    Okno.DisplayData("IN " + DateTime.Now + ": " + udalost.Id);
                }
                catch (Exception)
                {

                }
            }
            zprava = VychoziProtokol.ObsluzUdalost(udalost, hodnota, this);
            if (ukazovat && zprava != "")
            {
                try
                {
                    Okno.DisplayData(zprava);
                }
                catch (Exception)
                {

                }
            }
        }

        /// <summary>
        /// Odeslání příkazu pro dané zařízení (na jeho port). Příkaz má být bez koncového symbolu, protože se odesílá pomocí WriteLine.
        /// </summary>
        /// <param name="prikaz">Odesílaný příkaz (bez koncového symbolu)</param>
        public void OdesliPrikaz(string prikaz)
        {
            if (ukazovat)
            {
                try
                {
                    Okno.DisplayData("IN " + DateTime.Now + ": " + prikaz);
                }
                catch (Exception)
                {
                    
                }
            }
            try
            {
                Komunikace.WriteLine(prikaz);
            }
            catch (Exception ex)
            {
                if (ukazovat)
                {
                    try
                    {
                        Okno.DisplayData("Chyba odeslání příkazu: " + prikaz + " (" + ex.Message + ")");
                    }
                    catch (Exception)
                    {

                    }
                }
            }
        }

        /// <summary>
        /// Odeslání příkazu pro dané zařízení (na jeho port) s časovou prodlevou. Příkaz má být bez koncového symbolu, protože se odesílá pomocí WriteLine.
        /// </summary>
        /// <param name="prikaz">Odesílaný příkaz (bez koncového symbolu)</param>
        /// <param name="prodleva">Časová prodleva v ms. Prodleva je omezena intervalem 0..1000. Hodnoty mimo interval se korigují na hraniční hodnotu.</param>
        public void OdesliPrikaz(string prikaz, int prodleva)
        {
            prodleva = Math.Max(0, Math.Min(prodleva, 1000));
            System.Threading.Thread.Sleep(prodleva);
            this.OdesliPrikaz(prikaz);
        }
    }
}
