﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CPXDI_model
{
    public partial class ZarizeniForm : Form
    {
        public ZarizeniForm(Zarizeni zarizeni)
        {
            InitializeComponent();

            aktZarizeni = zarizeni;

            this.Text = "Log: " + aktZarizeni.Nazev;
            logLabel.Text = "Log komunikace zařízení: " + aktZarizeni.Nazev;
            if (zarizeni.VychoziProtokol != null)
            {
                slovnikPrikazy = zarizeni.VychoziProtokol.DefinujPrikazyOvladani();
            }
            if (slovnikPrikazy == null)
            {
                slovnikPrikazy = new Dictionary<string, string>();
            }
            if (!slovnikPrikazy.ContainsKey("<jiný příkaz>"))
            {
                slovnikPrikazy.Add("<jiný příkaz>", "");
            }
            prikazyListBox.DataSource = slovnikPrikazy.Keys.ToList();
        }

        Zarizeni aktZarizeni;
        Dictionary<string, string> slovnikPrikazy = new Dictionary<string, string>();

        /// <summary>
        /// Pro zobrazení zprávy do logu
        /// </summary>
        /// <param name="msg">zpráva k zobrazení do logu</param>
        public void DisplayData(string msg)
        {
            logTextBox.Invoke(new EventHandler(delegate
            {
                logTextBox.AppendText(msg + "\r\n");
            }));
        }

        /// <summary>
        /// Při změně vybraného příkazu v listBoxu změní text příkazu v textBoxu níže.
        /// </summary>
        private void prikazyListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            prikazTextBox.Text = slovnikPrikazy[(string)prikazyListBox.SelectedItem];
        }

        /// <summary>
        /// Tlačítkem odešle vyplněný příkaz do zařízení
        /// </summary>
        private void OdeslatButton_Click(object sender, EventArgs e)
        {
            aktZarizeni.OdesliPrikaz(prikazTextBox.Text);
        }

        /// <summary>
        /// Nastaví CheckBox pro Timer dle parametru (používat / nepoužívat)
        /// </summary>
        /// <param name="pouzivat">Indikuje, jestli se má Timer používat, či ne</param>
        public void NastavTimerCheckBox(bool pouzivat)
        {
            timerCheckBox.Enabled = pouzivat;
            timerCheckBox.Checked = pouzivat;
        }

        /// <summary>
        /// Nastaví CheckBox pro posílání neexistujících hodnot požadovaných veličin zařízení dle parametru (posílat / neposílat)
        /// </summary>
        /// <param name="posilat">Indikuje, jestli se mají neexistující hodnoty posílat, či ne</param>
        public void NastavNeexHodCheckBox(bool posilat)
        {
            neexHodCheckBox.Checked = posilat;
        }

        private void timerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            aktZarizeni.pouzivatTimer = timerCheckBox.Checked;
        }

        private void neexHodCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            aktZarizeni.posilatNeexistujiciHodnoty = neexHodCheckBox.Checked;
        }

        private void ZarizeniForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            aktZarizeni.ukazovat = false;
        }
    }
}
